<?php
require_once('squal_pre.php');
require_once('common/frs/FRSFile.class');
require_once('common/frs/FRSRelease.class');
require_once('common/frs/FRSPackage.class');
require_once('common/tracker/ArtifactType.class');
require_once('common/tracker/Artifact.class');
require_once('common/tracker/ArtifactExtraField.class');
require_once('common/tracker/ArtifactExtraFieldElement.class');

define ('GREEN', "\033[01;32m" );
define ('NORMAL', "\033[00m" );
define ('RED', "\033[01;31m" );


/**
 * Dowload the latest version of GForge 4.x
 */
function downloadLatestVersion() {
	$basepath = getcwd();
	$filename = '/gforge-4.6rc1.tar.bz2';
	$download_url = "http://gforge.org/gf/download/frsrelease/157/252/".$filename;
	$fullpath = $basepath.'/'.$filename;
	
	if (!file_exists($fullpath) || filesize($fullpath) != 3326353) {
		$output = array();
		show("Downloading $download_url ...");
		$error = execute("wget $download_url -O $fullpath", $output);
		if ($error || !file_exists($fullpath) || filesize($fullpath) != 3326353) {
			show(RED."Couldn't fetch $download_url using wget. Aborting conversion.".NORMAL);
			die(1);
		}
	}
	
	// Unpack the file
	show("Unpacking the archive...");
	$error = execute("tar jxf $fullpath", $output);
	
	// The file gets extracted to $basepath/gforge-4.6rc1
	$GLOBALS['LATEST_GFORGE_PATH'] = $basepath."/gforge-4.6rc1";
}

function backupDatabase() {
	global $sys_dbname;

	show(RED."WARNING - The conversion process will modify your existing database. This could cause irreversible
problems in the case the conversion goes wrong. You MUST make a backup of your database before upgrading
to GForge 5. To backup your database, execute the following command:
	pg_dump -U postgres ".$sys_dbname." > gforgedb_backup.sql".NORMAL);
	
	while (true) {
		show(RED.'Do you want to continue? (y/N)'.NORMAL);
		$proceed = strtolower(trim(fgets(STDIN)));
		
		if ($proceed === '' || $proceed == 'n' || $proceed == 'no') {
			show(RED.'Conversion aborted'.NORMAL);
			exit(1);
		} elseif ($proceed == 'y' || $proceed == 'yes') {
			break;
		}
	}

}

function getIncludePath() {
	$gfp = $GLOBALS['LATEST_GFORGE_PATH'];
	return ".:$gfp:$gfp/www/include:/etc/gforge";
}

/**
 * Try to find out the database startpoint using the existing .sql files
 */
function createStartpoint() {
	// Create the startpoint table (if it doesn't exit)
	$res = db_query("SELECT COUNT(*) AS exists FROM pg_class WHERE relname = 'database_startpoint' AND relkind = 'r'");
	if (!$res) { // db error
		show(RED.'Error creating the startpoint table (1): '.db_error().NORMAL);
		exit(1);
	}
	if (db_result($res, 0, 'exists') == '0') {
		// Table doesn't exist. Create it.
		show('Creating startpoint table...');
		$res1 = db_query("CREATE TABLE database_startpoint (db_version varchar(10), db_start_date int)");
		$res2 = db_query("CREATE TABLE database_changes (filename text)");
		if (!$res1 || !$res2) { // db error
			show(RED.'Error creating the startpoint table (2): '.db_error().NORMAL);
			exit(1);
		}
	}
	
	// If the table exists and it already has records, don't try to autodetect the version
	$res = db_query("SELECT COUNT(*) AS exists FROM database_startpoint");
	if ((int)db_result($res, 0, 'exists') > 0) {
		return;
	}
	
	// We need to detect the latest script inserted into the DB.
	show('Autodetecting version...');


	$versions = array('2.5', '2.6','3.0','3.1','3.21','3.3','4.0','4.0.1','4.0.2','4.5','4.5.3','4.5.6','4.5.11');
	$dates    = array(20001219,20010601,20030701,20031026,20040108,20040326,20041025,20041107,20041215,
		'20050803',20050807,20050809,20050810);
	$files = array('20001219-sourceforge-2.5','20010601-sourceforge-2.6','20030701-gforge-3.0','20031026-gforge-3.1',
		'20040108-gforge-3.21','20040326-gforge-3.3','20041025-gforge-4.0','20041107-gforge-4.0.1','20041215-gforge-4.0.2',
		'20050803-gforge-4.5','20050807-gforge-4.5.3','20050809-gforge-4.5.6','20050810-gforge-4.5.11');
	
	// Paths where the SQL files may be
	$possible_sql_paths = array('/opt/gforge/db', '/opt/gforge/gforge/db', $sys_plugins_path.'../db', $sys_urlroot.'../db');
	// Also use the include_path
	$include_paths = explode(':', ini_get('include_path'));
	foreach ($include_paths as $include_path) {
		$possible_sql_paths[] = $include_path.'/db';
	}
	
	// Check the correct path
	$sql_path = '';
	foreach ($possible_sql_paths as $possible_sql_path) {
		if (is_dir($possible_sql_path)) {
			$sql_path = $possible_sql_path;
			break;
		}
	}
	
	if (!$sql_path) {
		show(RED.'The GForge .sql files coulnd\'t be found. Aborting...'.NORMAL);
		exit(1);
	}
	
	show("GForge SQL files found in $sql_path...");
	
	// Now search for the latest script
	$version = '(unknown)';
	$dh = opendir($sql_path);
	$latest_date = 0;
	while ($file = readdir($dh)) {
		// Check if it's an upgrade script
		$date = substr($file, 0, 8);
		if (intval($date) <= 20000000) {
			continue;
		}
		
		$date = intval($date);
		$latest_date = max($date, $latest_date);
	}
	closedir($dh);

	if (!$latest_date) {
		show(RED.'Couldn\'t autodetect GForge version. Aborting...'.NORMAL);
	}
	
	$date = $latest_date;
	
	// See if the latest script corresponds to a known version. This is not strictly necessary.
	if ( ($i = array_search($latest_date, $dates)) !== false) {
		$version = $versions[$i];
	} else {
		// The latest file does not correspond to a known version. Maybe used SVN?
		$version = '(unknown)';
	}
	
	show("Found version $version (date: $date)");
	
	db_query("INSERT INTO database_startpoint (db_version, db_start_date) VALUES ('$version', '$date')");
}

/**
 * Run the db-upgrade script and perform other queries
 */
function runDbUpgrade() {
	global $sys_dbuser, $sys_dbname;

	$php_include_path = getIncludePath();
	$gforge_path = $GLOBALS['LATEST_GFORGE_PATH'];
	$command = "cd $gforge_path/db && php -d include_path=$php_include_path upgrade-db.php";
	$output = array();
	
	show('Upgrading the database...');
	$error = execute($command, $output);
	
	show('Setting correct database permissions...');
	$error = execute('psql -U postgres template1 -c "ALTER USER '.$sys_dbuser.' CREATEUSER"', $output);
	if ($error) {
		show(RED.'Couldn\'t give admin privileges to database user '.$sys_dbuser.'. Please make sure the user "postgresql" is able to connect to the database and run the upgrade script again.'.NORMAL);
		exit(1);
	}
	
	show('Dropping obsolete tables...');
	db_query('DROP TABLE artifact_category CASCADE');		// Note: may cause errors if the table doesn't exist, but it's not fatal
	db_query('DROP TABLE artifact_group CASCADE');
	db_query('DROP TABLE artifact_resolution CASCADE');
	execute('psql -U postgres '.$sys_dbname.' < '.$gforge_path.'/plugins/scmsvn/db/scmsvn-init.sql', $output);
	
	show('Running misc. queries...');
	db_query('UPDATE mail_group_list SET status=0 WHERE status=9;');
	db_query('UPDATE user_group SET role_id=(SELECT role_id FROM role WHERE role_name=\'Admin\' AND group_id=user_group.group_id) WHERE role_id=1;');


}

function runConversion() {
	// Create the directory where the tables will be dumped with the proper permissions
	$output = array();
	exec('rm -rf /tmp/gf5 && mkdir /tmp/gf5 && chown -R postgres:postgres /tmp/gf5');

	// Now run the conversion script
	show('Running conversion script...');
	$php_include_path = getIncludePath();
	$command = "php -d include_path=$php_include_path conversion.php";
	exec($command, $output);
	show(implode("\n", $output));
	
	/*// Create the "gforge5" database
	$output = array();
	$error = exec('createdb -U gforge --encoding UNICODE gforge5 2>&1', $output);
	if ($error && $error != 'CREATE DATABASE') {
		show(RED.'Error creating the "gforge5" database:'.implode("\n", $output).NORMAL);
		exit(1);
	}
	
	// There's no way of catching errors in these commands... for some reason exec() returns
	// the latest string instead of the return code
	$output = array();
	exec('psql -U postgres gforge5 < gforge5.sql', $output);
	exec('psql -U postgres gforge5 < import.sql', $output);
	
	show('Note: you can safely ignore errors about "activity_log" and "cron_history"');*/
}

/**
 * Execute a command with return code
 */
function execute($command, &$output, $explode_output=true) {
	$retval = 0;
	if ($explode_output) {
		exec($command, $output, $retval);
	} else {
		$output = "";
		$ph = popen($command, 'r');
		while ($buffer = fread($ph, 4096)) {
			$output .= $buffer;
		}
		pclose($ph);
		$retval = 0;		// No way to catch errors with popen :(
	}
	return $retval;		
}

function show($text, $newLine = true) {
	if ($newLine) {
		$text = GREEN.$text .NORMAL."\n";
	}
	fwrite(STDOUT, $text);
}
	
function inputText() {
	return trim(fgets(STDIN));
}

/*****************************
	Main script
******************************/
db_connect();

backupDatabase();
downloadLatestVersion();
createStartpoint();
runDbUpgrade();
runConversion();

show('The database has been dumped to /tmp/gf5. Please, copy this directory to the GForge 5 server and run the upgrade scripts.');
?>
