CREATE OR REPLACE FUNCTION getAllUserProjectsIds(user_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	project_row RECORD;
	project_sql VARCHAR;
	sql VARCHAR;
BEGIN
	IF user_id IS NULL THEN RETURN; END IF;

	-- This code is copied from User::getAllProjects()
	project_sql := \' \';
	
	FOR project_row IN SELECT p.project_id, p.root_path FROM project p, user_project up
					WHERE (up.user_id=user_id OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=user_id))
					AND up.project_id=p.project_id AND p.root_parent_project_id > 0 LOOP
		project_sql := project_sql || \' OR (p.root_path ~ \'\'^\' || project_row.root_path || \',\'\') \';
	END LOOP;
	
	-- Get all the child projects
	sql := \'SELECT DISTINCT p.project_id, p.root_path FROM project p, user_project up
					WHERE ((up.user_id=\' || user_id || \' OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=\' || user_id || \'))
					AND up.project_id=p.project_id) \' || project_sql;
	
	FOR project_row IN EXECUTE(sql) LOOP
		RETURN NEXT project_row.project_id;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;

CREATE OR REPLACE FUNCTION getAllProjectUsersIds(project_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	user_row RECORD;
	project_ids VARCHAR;
	root_path VARCHAR;
	sql VARCHAR;
BEGIN
	IF project_id IS NULL THEN RETURN; END IF;
	-- Code copied from Project::getMembers()
	SELECT p.root_path INTO root_path FROM project p WHERE p.project_id=project_id;
	
	IF root_path IS NOT NULL THEN
		project_ids := \' up.project_id IN (\' || root_path || \')\';
	ELSE
		project_ids := \' up.project_id=\' || project_id;
	END IF;
	
	sql := \'SELECT DISTINCT u.user_id FROM "user" AS u WHERE u.user_id IN (
						SELECT u2.user_id FROM user_project AS up, "user" as u2
						WHERE \' || project_ids || \' AND u2.user_id=up.user_id AND
						u2.is_group=false
					UNION
						SELECT ug.member_user_id AS user_id
						FROM usergroup_user AS ug, user_project AS up
						WHERE \' || project_ids || \' AND up.user_id=ug.usergroup_id
				)\';
	
	FOR user_row IN EXECUTE(sql) LOOP
		RETURN NEXT user_row.user_id;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;

-- Get all the projects the user has SCM read access to
CREATE OR REPLACE FUNCTION getAllUserProjectsIdsSCM(user_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	project_row RECORD;
	project_sql VARCHAR;
	root_path VARCHAR;
	sql VARCHAR;
BEGIN
	IF user_id IS NULL THEN RETURN; END IF;

	-- This code is copied from User::getAllProjects()
	project_sql := \' \';
	
	FOR project_row IN SELECT p.project_id, p.root_path FROM project p, user_project up
					WHERE (up.user_id=user_id OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=user_id))
					AND up.project_id=p.project_id AND p.root_parent_project_id > 0 LOOP
		project_sql := project_sql || \' OR (p.root_path ~ \'\'^\' || project_row.root_path || \',\'\') \';
	END LOOP;
	
	-- Get all the child projects
	sql := \'SELECT up.project_id,MAX(rs.value) AS max_value FROM project p, user_project up, user_project_role upr, role_setting rs
			WHERE (
				(up.user_id=\' || user_id || \' OR up.user_id IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=\' || user_id || \'))
				AND up.project_id=p.project_id AND upr.user_project_id=up.user_project_id AND rs.role_id=upr.role_id
				AND rs.section=\'\'scm\'\'
			) \' || project_sql || \'
			GROUP BY up.project_id
			\';
	
	FOR project_row IN EXECUTE(sql) LOOP
		IF project_row.max_value >= 0 THEN
			RETURN NEXT project_row.project_id;
		END IF;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;

-- From all the users in the project, retrieve only the ones that have at least SCM read permissions
CREATE OR REPLACE FUNCTION getAllProjectUsersIdsSCM(project_id INTEGER) RETURNS SETOF INTEGER AS E'
DECLARE
	user_ids RECORD;
	project_ids VARCHAR;
	root_path VARCHAR;
	sql VARCHAR;
	scm_settings_array RECORD;
BEGIN
	IF project_id IS NULL THEN RETURN; END IF;

	-- Code copied from Permission::getPermission()
	SELECT p.root_path INTO root_path FROM project p WHERE p.project_id=project_id;
	
	IF root_path IS NOT NULL THEN
		project_ids := \' IN (\' || root_path || \')\';
	ELSE
		project_ids := \'=\' || project_id;
	END IF;
	
	FOR user_ids IN SELECT * FROM getAllProjectUsersIds(project_id) LOOP
		sql := \'
			SELECT max(value) AS scm_setting FROM role_setting
			NATURAL JOIN user_project_role NATURAL JOIN user_project
			WHERE role_setting.section=\'\'scm\'\' AND role_setting.ref_id=0
			AND (user_project.user_id=\' || user_ids.getallprojectusersids || \' OR user_project.user_id IN 
				(SELECT usergroup_id FROM usergroup_user WHERE member_user_id=\' || user_ids.getallprojectusersids || \'))
			AND user_project.project_id \' || project_ids || \'
		\';
		FOR scm_settings_array IN EXECUTE(sql) LOOP
			IF scm_settings_array.scm_setting >= 0 THEN
				RETURN NEXT user_ids.getallprojectusersids;
			END IF;
		END LOOP;
	END LOOP;
END
'
LANGUAGE plpgsql
STABLE;

CREATE OR REPLACE VIEW nss_passwd_vw AS
	SELECT
		u.user_id,
		COALESCE(uu.unix_uid, u.user_id+20000) AS uid,		-- + UserUnixPeer::UNIX_UID_ADD
		COALESCE(uu.unix_gid, u.user_id+20000) AS gid,		-- + UserUnixPeer::UNIX_GID_ADD 
		u.unix_name AS username,
		u.password_crypt AS passwd,
		u.firstname || ' ' || lastname AS gecos,
		'/home/' || u.unix_name AS homedir,
		COALESCE(us.path, '/bin/cvssh.pl') AS shell
	FROM "user" u
		LEFT OUTER JOIN user_unix uu ON u.user_id=uu.user_id
		LEFT OUTER JOIN unix_shell us ON uu.unix_shell_id=us.unix_shell_id
	WHERE u.status=1 AND u.is_group=false AND u.user_id != 100
;

CREATE OR REPLACE VIEW nss_group_vw AS
	SELECT
		p.unix_name AS groupname,
		'x'::VARCHAR AS passwd,
		p.project_id + 50000 AS gid,						-- + ProjectScm::UNIX_GROUP_ADD
		(p.is_public AND COALESCE(pscm.enable_anonread, false)) AS has_anonymous,
		(NOT p.is_public OR NOT COALESCE(pscm.enable_anonread, true)) AS has_apache
	FROM project p 
		LEFT OUTER JOIN project_scm pscm ON pscm.project_id=p.project_id
	WHERE p.status=1
	UNION
	SELECT
		u.unix_name AS groupname,
		'x'::VARCHAR as passwd,
		u.user_id + 20000 AS gid,
		false AS has_anonymous,
		false AS has_apache
	FROM "user" u
	WHERE u.status=1 AND u.is_group=false AND u.user_id != 100
;

UPDATE user_unix SET unix_gid=unix_uid;
