<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ForumAdminFormRenderer.php 417 2006-02-10 19:33:48Z marcelo $
 * @author	Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('SystemUtils.php');
require_once('GFConf.php');
require_once('GFUrl.php');
require_once('session/Session.php');
require_once('html/Paginator.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumThread.php');
require_once('plugins/forum/lib/Forum.php');

/**
 * The forums will be re-used in another section, so we wrap up all the pages in this
 * class so we can use them across the sections.
 */
class ForumFormRenderer {
	/** Flat: one thread/message after another */
	const THREAD_STYLE_FLAT = 'flat';
	/** Nested: Messges in thread displayed in tree-fashion (title only) */
	const THREAD_STYLE_NESTED = 'nested';
	/** Threaded: Messges in thread displayed in tree-fashion (title + text) */
	const THREAD_STYLE_THREADED = 'threaded';
	
	/** Flat message: used when displayed in thread */
	const MESSAGE_STYLE_FLAT = 1;
	/** Reply message: used when replying a msesage*/
	const MESSAGE_STYLE_REPLY = 2;
	
	private $section = null;
	private $ref_id = null;
	private $wwwlibPath;
	private $template;
	private $language;
	private $errors = array();
	private $show_only_latest = true;
	private $readonly = false;

	public function __construct($section, $ref_id, $template, $language) {
		$this->section = $section;
		$this->ref_id = $ref_id;
		$this->template = $template;
		$this->language = $language;
		
		$this->wwwlibPath = SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/forum/wwwlib/forms');
	}
	
	/**
	 * Returns a table with all the forums
	 *
	 * @return string
	 */
	public function showForumBrowseForm() {
		$Lang = $this->language;
		$theme = $this->template;
		$url = GFUrl::getUrl();

		// Retrieve the objects array
		$crit = new Criteria();
		$crit->add(ForumPeer::SECTION, $this->section);
		$crit->add(ForumPeer::REF_ID, $this->ref_id);
		$arr = ForumPeer::doSelect($crit);
		
		$ret = '';
		
		if (count($arr) < 1) {
			$ret .= '<span class="nonefound">'.$Lang->getText('Forum.NoneFound').'</span>';
		} else {
			$theader = array();
		
			$theader[] = $Lang->getText('Forum.ForumNameName');
			$theader[] = $Lang->getText('Forum.DescriptionName');
			$theader[] = $Lang->getText('Forum.ThreadCountName');
			$theader[] = $Lang->getText('Forum.MessageCountName');
			$theader[] = $Lang->getText('Forum.MostRecentDateName');
			$theader[] = $Lang->getText('Forum.LatestThreads');
		
			$ret .= $theme->tableTop($theader);
		
			foreach ($arr as $item) {
				if ($item->userCanRead()) {
					$trow = array();
					$tmod = array();
					
					$top_threads = ForumThreadPeer::getTopThreads($item);
					$num_threads = count($top_threads);
					
					if ($num_threads > 0) {
						$trow[] = '<a href="'. GFUrl::getUrl()->currentUrl(array('action' => 'ForumBrowse', 'forum_id' => $item->getForumId())).'"><img src="/images/forum_icon.gif" height="16" width="16" border="0" alt="" />&nbsp;'.$item->getForumName().'</a>';
						$tmod[] = 'rowspan="'.$num_threads.'" valign="middle" width="25%"';
						$trow[] = $item->getDescription();
						$tmod[] = 'rowspan="'.$num_threads.'" valign="middle"';
						$trow[] = $item->getThreadCount();
						$tmod[] = 'rowspan="'.$num_threads.'" valign="middle"';
						$trow[] = $item->getMessageCount();
						$tmod[] = 'rowspan="'.$num_threads.'" valign="middle"';
						$trow[] = $item->getMostRecentDate();
						$tmod[] = 'rowspan="'.$num_threads.'" valign="middle"';

						$thread = $top_threads[0];
						$trow[] = '1. <a href="'.$url->currentUrl(array('action' => 'ForumBrowse', '_forum_action' => 'ForumMessageBrowse', 'thread_id' => $thread->getForumThreadId())).'">'.$thread->getThreadName().'</a>';
						$tmod[] = 'bgcolor="#ffffff"';
						$ret .= $theme->tableRow($trow, $tmod);
						
						for ($i=1; $i < $num_threads; $i++)  {
							$thread = $top_threads[$i];
							$trow = array();
							$tmod = array();
							$trow[] = ($i+1).'. <a href="'.$url->currentUrl(array('action' => 'ForumBrowse', '_forum_action' => 'ForumMessageBrowse', 'thread_id' => $thread->getForumThreadId())).'">'.$thread->getThreadName().'</a>';
							$tmod[] = 'bgcolor="#ffffff"';
							$ret .= $theme->tableRow($trow, $tmod);
						}
					
					} else {
						$trow[] = '<a href="'. GFUrl::getUrl()->currentUrl(array('action' => 'ForumBrowse', 'forum_id' => $item->getForumId())).'"><img src="/images/forum_icon.gif" height="16" width="16" border="0" alt="" />&nbsp;'.$item->getForumName().'</a>';
						$tmod[] = 'width="25%"';
						$trow[] = $item->getDescription();
						$tmod[] = '';
						$trow[] = $item->getThreadCount();
						$tmod[] = '';
						$trow[] = $item->getMessageCount();
						$tmod[] = '';
						$trow[] = '-';
						$tmod[] = '';
						$trow[] = $Lang->getText('Forum.NoThreads');
						$tmod[] = '';
						$ret .= $theme->tableRow($trow, $tmod);
					}
				}
			}
		
			$ret .= $theme->tableBottom();
		}
		return $ret;
	}
	
	/**
	 * Displays a table with all the threads. According to user selection, the threads
	 * can be displayed in a threaded, nested or flat style
	 */
	public function getThreads($forum) {
		$Lang = $this->language;
		$theme = $this->template;
		$session = Session::getSession();
		$url = GFUrl::getUrl();
		
		if (!$forum instanceof Forum) {
			$forum_id = $forum;
			// first make sure the forum is correct
			$forum = ForumPeer::retrieveByPk($forum_id);
			if (!is_object($forum) || ($forum->getSection() != $this->section) || ($forum->getRefId() != $this->ref_id)) {
				throw new PermissionDeniedException('Invalid forum ID: '.$forum_id.' for section "'.$this->section.'" and reference ID '.$this->ref_id);
			}
		} else {
			$forum_id = $forum->getForumId();
		}
		
		// see if the user chose to save the current date
		if (isset($_GET['forum_save']) && !is_null($user = $session->getUser())) {
			$forum->setSavedDate($user);
		}

		$output = '';
		$output .= $this->showBrowseOptions();
		
		// see how do we want to display the threads
		$style = $this->getThreadStyle();
		if ($style == self::THREAD_STYLE_NESTED) {
			// Nested messages
			$output .= $this->showNestedMessages($forum);
			return $output;
		} else if ($style == self::THREAD_STYLE_THREADED) {
			// Threaded messages
			$output .= $this->showThreadedMessages($forum);
			return $output;
		}
		
		// If we are here, it means we want to see the flat list of threads
		
		$saved_date = $forum->getSavedDate($session->getUser());
		
		// Retrieve the objects array, paginated
		$limit = $this->getThreadLimit();
		
		if ($this->show_only_latest) {
			$arr = $forum->getLatestThreads(new Criteria());
		} else {
			$c = new Criteria();
			$c->add(ForumThreadPeer::FORUM_ID, $forum->getForumId());
			$c->addDescendingOrderByColumn(ForumThreadPeer::MOST_RECENT_DATE);
			$arr = ForumThreadPeer::doSelect($c);
		}
		$total = count($arr);

		$paginator = new Paginator('forum_thread', $limit, $total);

		$theader = array();
	
		$theader[] = $Lang->getText('ForumThread.ThreadNameName');
		$theader[] = $Lang->getText('ForumThread.CreatedByName');
		$theader[] = $Lang->getText('ForumThread.MessageCountName');
		$theader[] = $Lang->getText('ForumThread.MostRecentDateName');
	
		$output .= $theme->tableTop($theader);
	
		if (count($arr) == 0) {
			$output .= $theme->tableFullRow('<span class="nonefound">'.$Lang->getText('ForumThread.NoneFound').'</span>');
		} else {
			foreach ($arr as $item) {
				$trow = array();
				$unread = ($saved_date < $item->getMostRecentDate(null));
				if ($unread) {
					$trow[]='<strong><a href='.$url->currentUrl(array('_forum_action' => 'ForumMessageBrowse', 'thread_id'=>$item->getForumThreadId())).'> '.$item->getThreadName().'</a></strong>';
				} else {
					$trow[]='<a href='.$url->currentUrl(array('_forum_action' => 'ForumMessageBrowse', 'thread_id'=>$item->getForumThreadId())).'> '.$item->getThreadName().'</a>';
				}
				
				$user = $item->getUser();
				if ($user->isUndefinedUser()) {
					// undefined user?
					$trow[] = $item->getUndefinedUserName();
				} else {
					$trow[] = $user->getFullName();	// retrieves the created_by field
				}
				$trow[] = $item->getMessageCount();
				$trow[] = $item->getMostRecentDate();
		
				$output .= $theme->tableRow($trow);
			}
		}
		
		$output .= $theme->listPaginatorRow($paginator);		
		$output .= $theme->tableBottom($this->getForumButtons($forum));
		
		return $output;
	}

	/**
	 * Returns true if the form was processed and errors were found
	 * @return bool
	 */
	public function hasErrors() {
		return (count($this->errors) > 0);
	}
	
	/**
	 * Returns an array with the errors found after processing the POST data
	 * @return array
	 */
	public function getErrors() {
		return $this->errors;
	}

	/**
	 * Returns a form for creating a new thread
	 *
	 * @return	strings
	 */
	public function newThreadForm($Forum, $redirect_to, $force_form=false) {
		if ($this->readonly) {
			throw new PermissionDeniedException();
		}

		if ($Forum instanceof Forum) {
			$forum_id = $Forum->getForumId();
		} else {
			$forum_id = $Forum;
			$Forum = ForumPeer::retrieveByPk($forum_id);
		}
		
		
		$theme = $this->template;
		$Lang = $this->language;
		$session = Session::getSession();

		if (!is_object($Forum)) {
			throw new PermissionDeniedException();
		} else if ($Forum->getSection() != $this->section || $Forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidRefId'));
		} else if (!$session->isLoggedIn() || !$Forum->userCanPost($session->getUser())) {
			throw new PermissionDeniedException();
		}

		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_thread_add_submit']) && !$force_form) {
			return $this->postNewThread($Forum, $redirect_to);
		}
		
		$thread_name = ((isset($_POST['thread_name'])) ? htmlspecialchars($_POST['thread_name']) : '' );
		$message = ((isset($_POST['message'])) ? htmlspecialchars($_POST['message']) : '' );

		$formFile = SystemUtils::buildFilePath($this->wwwlibPath, 'ForumThreadAdd-tpl.php');
		ob_start();	
		include($formFile);
		$form = ob_get_clean();
		
		return $form;
	}
	
	private function postNewThread(Forum $forum, $redirect_to) {
		if ($this->readonly) {
			throw new PermissionDeniedException();
		}

		try {
			$thread_name = Validator::sanitizeString($_POST['thread_name']);
			$message = Validator::sanitizeString($_POST['message']);
			$forum->startThread($thread_name, $message);
		} catch (InvalidDataException $ex) {
			$this->errors = $ex->getErrors();
			return $this->newThreadForm($forum, $redirect_to, true);
		}
		
		GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('ForumThread.AddFeedback')));
	}
	
	/**
	 * Checks that the tread is valid inside this forum, and returns the thread object.
	 *
	 * @param	int $thread_id
	 * @return	ForumThread
	 */
	private function validateThread($thread_id) {
		$c = new Criteria();
		$c->add(ForumThreadPeer::FORUM_THREAD_ID, $thread_id);
		$thread = ForumThreadPeer::doSelectJoinForum($c);
		
		if (count($thread) == 0) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidThreadID'));
		}
		
		$thread = $thread[0];
		
		$ref_id = $thread->getForum()->getRefId();
		$section = $thread->getForum()->getSection();
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($this->language->getText('ForumThread.InvalidRefId'));
		}
		
		return $thread;
	}
	
	/**
	 * Displays the messages inside a thread.
	 */
	public function getThreadMessages($thread) {
		$theme = $this->template;
		$Lang = $this->language;
		
		if (!($thread instanceof ForumThread)) {
			// $thread is the thread_id instead of the thread object
			$thread = $this->validateThread($thread);
		}
		
		$output = $this->showBrowseOptions();
		
		$style = $this->getThreadStyle();
		
		switch ($style) {
			default: 
			case ForumFormRenderer::THREAD_STYLE_FLAT: {
				$output .= $this->showFlatMessages($thread);
				break;
			}
			case ForumFormRenderer::THREAD_STYLE_NESTED: {
				$output .= $this->showNestedMessages($thread->getForum());
				break;
			}
			case ForumFormRenderer::THREAD_STYLE_THREADED: {
				$output .= $this->showThreadedMessages($thread->getForum());
				break;
			}
		}
		return $output;
	}
	
	private function showFlatMessages(ForumThread $thread) {
		$session = Session::getSession();
		$Lang = $this->language;
		$forum = $thread->getForum();
		
		$saved_date = $forum->getSavedDate($session->getUser());
		$output = '';

		$messages = $thread->getThreadMessagesFlat();
		foreach ($messages as $message) {
			$output .=  $this->getMessage($message);
			$output .= '<br />'."\n";
		}
		
		return $output;
	}
	
	private function getMessage(ForumMessage $message, $style = ForumFormRenderer::MESSAGE_STYLE_FLAT) {
		$Lang = $this->language;
		$url = GFUrl::getUrl();
		$user = $message->getUser();
		$session = Session::getSession();
		$theme = $session->getTheme();
		$forum = $message->getForumThread()->getForum();
		
		$return = '';
		
		$message_header = $Lang->getText('ForumMessage.By').': ';
		if ($user->isUndefinedUser()) {
			$message_header .= $message->getMailSender();
		} else {
			$message_header .= '<a href="'.$url->userUrl($user).'">'.$user->getFullName().'</a>';
		}		
		$message_header .= '<br />';
		
		if (false) {		// check if we have attachments
			//@TODO
		} else {
			$message_header .= $Lang->getText('ForumMessage.NoAttachments').'<br />';
		}
		
		$message_header .= '<a href="'.$url->currentUrl(array('_forum_action' => 'MessageReply', 'message_id'=>$message->getForumMessageId())).'"> '.$message->getSubject();
		if (!$this->readonly && $session->isLoggedIn() && $forum->userCanPost($session->getUser())) {
			$message_header .= ' ['.$Lang->getText('ForumMessage.Reply').']';
		}
		$message_header .= '</a><br />';
		
		$message_header .= $message->getPostDate().'<br />';
		
		if ($style == ForumFormRenderer::MESSAGE_STYLE_FLAT) {
			$return .= '
				<table border="0">
					<tr><td bgcolor="'.GFTheme::COLOR_ROW_EVEN.'" nowrap="nowrap">';
			$return .= $message_header;
			
			$return .= '</td></tr>
					<tr><td>';
	
			$return .= nl2br($message->getBody());
	
			$return .= '	</td></tr>
				</table>';
		} elseif ($style == ForumFormRenderer::MESSAGE_STYLE_REPLY) {
			$return .= $theme->tableTop(array($Lang->getText('ForumMessage.ReplyTitle', array($message->getForumMessageId()))));
			$return .= $theme->tableRow(array($message_header));
			$return .= $theme->tableRow(array(nl2br($message->getBody())));
			$return .= $theme->tableBottom();
		}
			
		return $return;
	}

	/**
	 * Checks that the message is valid inside this forum, and returns the ForumMessage object
	 * @param	int $message_id
	 * @return	ForumMessage
	 */
	private function validateMessage($message_id) {
		$c = new Criteria();
		$c->add(ForumMessagePeer::FORUM_MESSAGE_ID, $message_id);
		$messages = ForumMessagePeer::doSelectJoinForumThread($message_id);
		
		if (count($messages) == 0) {
			throw new PermissionDeniedException($Lang->getText('Forum.InvalidForumMessageId'));
		}
		
		$message = $messages[0];
		$forum = $message->getForumThread()->getForum();
		
		$ref_id = $forum->getRefId();
		$section = $forum->getSection();
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($Lang->getText('ForumMessage.InvalidRefId'));
		}
		
		return $message;
	}

	public function messageReplyForm($message, $redirect_to, $force_form=false) {
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$session = Session::getSession();

		if (!($message instanceof ForumMessage)) {
			$message = $this->validateMessage($message);
		}
		
		$forum = $message->getForumThread()->getForum();
		
		// checks that the user can post messages in the forum
		if (!is_object($forum)) {
			throw new PermissionDeniedException();
		} else if ($forum->getSection() != $this->section || $forum->getRefId() != $this->ref_id) {
			throw new PermissionDeniedException($lang->getText('ForumThread.InvalidRefId'));
		}
				
		// See if we've submitted the form. In that case, process the data.
		if (isset($_POST['forum_post_reply']) && !$force_form) {
			return $this->postMessageReply($message, $redirect_to);
		}

		$return = '';
		
		$return .= $this->getMessage($message, ForumFormRenderer::MESSAGE_STYLE_REPLY);
		$return .= '<br />';
		$return .= $this->getMessageThread($message);
		
		// If this is readonly, don't allow posting
		if ($this->readonly || !$session->isLoggedIn() || !$forum->userCanPost($session->getUser())) {
			return $return;
		}
		
		if (empty($_POST['subject'])) {
			$prefix = $lang->getText('ForumMessage.ReplyPrefix');
			$subject = $message->getSubject();
			if (preg_match('/^[[:blank:]]*'.strtolower($prefix).'/', strtolower($subject))) {
				$reply_subject = $subject;
			} else {
				$reply_subject = $prefix.' '.$subject;
			}
		} else {
			$reply_subject = htmlspecialchars($_POST['subject']);
		}
		
		if (!empty($_POST['body'])) {
			$reply_body = htmlspecialchars($_POST['body']);
		} else {
			$reply_body = '';
		}
		
		$return .= '<br />
			<form method="post">
			<p>
				<table border="0" align="center">
				<tr><td align="center">
				<h3>'.$lang->getText('ForumMessage.PostFollowup').'</h3>
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.SubjectName').':</strong><br />
					<input type="text" name="subject" value="'.$reply_subject.'" size="50" />
				</td></tr>
				<tr><td>
					<strong>'.$lang->getText('ForumMessage.BodyName').':</strong><br />
					<textarea name="body" rows="10" cols="50">'.$reply_body.'</textarea>
				</td></tr>
				<tr><td align="center">
					<?php echo HTMLWidget::button("submit",".$lang->getText(ForumMessage.PostReply).","forum_post_reply","","")?>
				</td></tr>
				</table>
			</p>
			</form>';
			
		return $return;
	}
	
	private function postMessageReply(ForumMessage $message, $redirect_to) {
		$session = Session::getSession();
		
		if ($this->readonly || !$session->isLoggedIn()) {
			throw new PermissionDeniedException();
		}
		
		try {
			$thread = $message->getForumThread();
			
			$subject = Validator::sanitizeString($_POST['subject']);
			$body = Validator::sanitizeString($_POST['body']);
			
			$new_message = new ForumMessage();
			$new_message->setUser($session->getUser());
			$new_message->setSubject($subject);
			$new_message->setBody($body);
			$new_message->setParentForumMessageId($message->getForumMessageId());
			
			$thread->addForumMessage($new_message);
			$thread->save();
		} catch (InvalidDataException $ex) {
			$this->errors = $ex->getErrors();
			return $this->messageReplyForm($message, $redirect_to, true);
		}
		
		GFUrl::redirect($redirect_to, array('feedback' => $this->language->getText('ForumMessage.AddFeedback')));
	}
	
	private function showBrowseOptions() {
		$lang = $this->language;
		$output = '';
		
		$output .= '<form method="get">';
		
		// we must pass all the _GET variables again
		foreach ($_GET as $var => $val) {
			if ($var == 'forum_thread_style' || $var == 'forum_thread_limit') continue;
			$output .= '<input type="hidden" name="'.$var.'" value="'.htmlspecialchars($val).'">'."\n";
		}
		
		$styles = array(
			self::THREAD_STYLE_FLAT => $lang->getText('ForumDisplayOptions.Flat'),
			self::THREAD_STYLE_NESTED => $lang->getText('ForumDisplayOptions.Nested'),
			self::THREAD_STYLE_THREADED => $lang->getText('ForumDisplayOptions.Threaded')
		);
		
		$default_style = $this->getThreadStyle();
		$output .= '<select name="forum_thread_style">'."\n";
		foreach ($styles as $style => $name) {
			if ($style == $default_style) {
				$selected = 'selected="selected"';
			} else {
				$selected = '';
			}
			$output .= '<option value="'.$style.'" '.$selected.'>'.$name.'</option>'."\n";
		}
		$output .= '</select>'."\n";
		
		$default_limit = $this->getThreadLimit();		
		$limits = array(25, 50, 75, 100);
		$output .= '<select name="forum_thread_limit">'."\n";
		foreach ($limits as $limit) {
			if ($limit == $default_limit) {
				$selected = 'selected="selected"';
			} else {
				$selected = '';
			}
			$output .= '<option value="'.$limit.'" '.$selected.'>'.$lang->getText('ForumDisplayOptions.Limit', array($limit)).'</option>'."\n";
		}
		$output .= '</select>'."\n";
		
		$output .= '&nbsp;&nbsp;';
		$output .= ' HTMLWidget::button("submit",$lang->getText('ForumDisplayOptions.ChangeView'),"forum_change_view","","") ';
		
		$output .= '</form>';
		$output .= '<br />'."\n";
		
		return $output;
	}
	
	private function getThreadStyle() {
		$user = Session::getSession()->getUser();

		if (isset($_REQUEST['forum_thread_style'])) {
			$style = $_REQUEST['forum_thread_style'];
			if (!is_null($user) && ($user->getPreference(Forum::PREFERENCE_THREAD_STYLE) != $style) ) {
				// Saves the new style as a preference for the user
				$user->setPreference(Forum::PREFERENCE_THREAD_STYLE, $style);
			} else {
				$_SESSION['forum_thread_style'] = $style;
			}
			return $style;
		} elseif (!is_null($user)) {
			$style = $user->getPreference(Forum::PREFERENCE_THREAD_STYLE);
			if (!$style) {
				return self::THREAD_STYLE_FLAT;
			} else {
				return $style;
			}
		} else if (isset($_SESSION['forum_thread_style'])) {
			return $_SESSION['forum_thread_style'];
		} else {
			return self::THREAD_STYLE_FLAT;
		}
	}

	private function getThreadLimit() {
		$user = Session::getSession()->getUser();
		$default_limit = 50;

		if (isset($_REQUEST['forum_thread_limit'])) {
			$limit = $_REQUEST['forum_thread_limit'];
			if (!is_null($user) && ($user->getPreference(Forum::PREFERENCE_THREAD_LIMIT) != $limit) ) {
				// Saves the new style as a preference for the user
				$user->setPreference(Forum::PREFERENCE_THREAD_LIMIT, $limit);
			}
			return $limit;
		} elseif (!is_null($user)) {
			$limit = $user->getPreference(Forum::PREFERENCE_THREAD_LIMIT);
			if (!$limit) {
				return $default_limit;
			} else {
				return $limit;
			}
		} else {
			return $default_limit;
		}
	}
	
	/**
	 * Show messages in a nested fashion (á la Slashdot)
	 */
	private function showNestedMessages(Forum $forum) {
		// retrieve all the forum messages in a tree fashion
		// (note that "threaded" and "nested" are pretty similar concepts, the
		// only difference is the way they are displayed)

		$theme = $this->template;
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$output = '';
		
		$limit = $this->getThreadLimit();
		
		if ($this->show_only_latest) {
			$msg_array = $forum->getLatestMessagesThreaded();
		} else {
			$msg_array = $forum->getAllMessagesThreaded();
		}
		
		if (count($msg_array) > 0) {
			$paginator = new Paginator('forum_thread', $limit, count($msg_array));
	
			// now show the messages
			$offset = $paginator->getOffset();
			for ($i=0; $i < $limit; $i++) {
				$idx = $offset+$i;
				if ($idx >= count($msg_array)) break;
				
				$message = $msg_array[$idx]["message"];
				$level = $msg_array[$idx]["level"];
	
				$output .= '<table border="0">'."\n";
				$output .= '<tr><td>'.str_repeat('&nbsp;', ($level*5)).'</td>';
				$output .= '<td>';
				$output .= $this->getMessage($message);
				$output .= '</td></tr>';
				$output .= '</table>'."\n";
			}
			
			$output .= '<div align="center">'.$paginator->getPaginator().'</div>';

			$output .= '<br />';
			// show the buttons -- we can't call $theme methods because we're not inside a table
			$output .= '<div align="center">';
			$buttons = $this->getForumButtons($forum);
			$i=0;
			foreach ($buttons as $link => $text) {
				$output .= '<a href="'.$link.'">'.$text.'</a>';
				$i++;
				if ($i < count($buttons)) {
					$output .= ' | ';
				}
			}
			$output .= "</div>";

		} else {
			// Wrap the message inside a table in order to be consistent
			//$output .= '<div align="center"><span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span></div>';
			$output .= $theme->tableTop(array(''));
			$output .= $theme->tableFullRow('<span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span>');
			$output .= $theme->tableBottom($this->getForumButtons($forum));
		}
		
		
		return $output;


	}

	private function showThreadedMessages(Forum $forum) {
		if ($this->show_only_latest) {
			$msg_array = $forum->getLatestMessagesThreaded();
		} else {
			$msg_array = $forum->getAllMessagesThreaded();
		}
		
		return $this->getThread($forum, $msg_array, true);
	}
	
	private function getThread(Forum $forum, $msg_array, $show_buttons=true, $selected_message=null) {
		$theme = $this->template;
		$lang = $this->language;
		$url = GFUrl::getUrl();
		
		$ret = '';
		
		$titles = array();
		$titles[] = $lang->getText('ForumDisplay.ThreadTitle');
		$titles[] = $lang->getText('ForumDisplay.AuthorTitle');
		$titles[] = $lang->getText('ForumDisplay.DateTitle');
		$ret .= $theme->tableTop($titles);
		
		$limit = $this->getThreadLimit();
		
		$saved_date = $forum->getSavedDate(Session::getSession()->getUser());

		if (count($msg_array) > 0) {
			$paginator = new Paginator('forum_thread', $limit, count($msg_array));
			
			// now show the messages
			$offset = $paginator->getOffset();
			for ($i=0; $i < $limit; $i++) {
				$idx = $offset+$i;
				if ($idx >= count($msg_array)) break;
				
				$message = $msg_array[$idx]["message"];
				$level = $msg_array[$idx]["level"];
	
				$cols = array();
				$msg_url = $url->currentUrl(array('_forum_action' => 'MessageReply', 'message_id' => $message->getForumMessageId()));
				$user = $message->getUser();
				
				$unread = ($saved_date < $message->getPostDate(null));
				
				// if the messge must be selected, display it in bold font instead of as a link
				if (!is_null($selected_message) && ($selected_message->getForumMessageId() == $message->getForumMessageId())) {
					$msg_text = $message->getSubject();
				} else {
					$msg_text = '<a href="'.$msg_url.'">'.$message->getSubject().'</a>';
				}
	
				if ($unread) {
					$cols[] = str_repeat('&nbsp;', ($level*5)).'<strong>'.$msg_text.'</strong>';
				} else {
					$cols[] = str_repeat('&nbsp;', ($level*5)).$msg_text;
				}
				
				if ($user->isUndefinedUser()) {
					$cols[] = $message->getMailSender();
				} else {
					$cols[] = '<a href="'.$url->userUrl($user).'">'.$message->getUser()->getFullName().'</a>';
				}
				$cols[] = $message->getPostDate();
				$ret .= $theme->tableRow($cols);
				
			}
			$ret .= $theme->listPaginatorRow($paginator);
		} else {
			$ret .= $theme->tableFullRow('<span class="nonefound">'.$lang->getText('ForumThread.NoneFound').'</span>');
		}
		
		if ($show_buttons) {
			$ret .= $theme->tableBottom($this->getForumButtons($forum));
		} else {
			$ret .= $theme->tableBottom();
		}
		
		return $ret;
	}

	private function getForumButtons(Forum $forum) {
		$session = Session::getSession();
		$loggedin = $session->isLoggedIn();
		$theme = $this->template;
		$Lang = $this->language;
		$url = GFUrl::getUrl();
		$forum_id = $forum->getForumId();
		
		if ($loggedin && $forum->userCanPost($session->getUser()) ) {
			$cancreate = true;
		} else {
			$cancreate = false;
		}

		$buttons = array();
	/*	
		if ($loggedin) {
			if ($forum->userIsMonitoring($session->getUser())) {
				$buttons[$url->getMonitorUrl('forum',$forum->getForumId(), $url->currentUrl(), false)] = $Lang->getText('Forum.StopMonitoring');
			} else {
				$buttons[$url->getMonitorUrl('forum',$forum->getForumId(), $url->currentUrl(), true)] = $Lang->getText('Forum.Monitor');
			}
			
			$buttons[$url->currentUrl(array('forum_save' => 1))] = $Lang->getText('Forum.SavePlace');
		}
	*/	
		if ($cancreate && !$this->readonly) {
			$buttons[$url->currentUrl(array('_forum_action' => 'ForumThreadAdd', 'forum_id'=>$forum_id))] = $Lang->getText('Forum.StartThread');
		}
		return $buttons;	

	}
	
	private function getMessageThread($message) {
		$threaded_thread = $message->getForumThread()->getThreadMessagesThreaded();
		$forum = $message->getForumThread()->getForum();
		
		return $this->getThread($forum, $threaded_thread, false, $message);
	}
	
	public function getHistory($forum) {
		$res = '';
		$lang = $this->language;
		$url = GFUrl::getUrl();
		$theme = Session::getSession()->getTheme();

		// Basic validation
		if (!($forum instanceof Forum)) {
			$forum = ForumPeer::retrieveByPk($forum);
			if (is_null($forum)) {
				throw new PermissionDeniedException($Lang->getText('Forum.InvalidForumID'));
			}
		} 
		
		$ref_id = $forum->getRefId();
		$section = $forum->getSection();
		$output = '';
		
		if ($ref_id != $this->ref_id || $section != $this->section) {
			throw new PermissionDeniedException($Lang->getText('ForumThread.InvalidRefId'));
		}
		
		// See if we must show a specific month
		if (isset($_GET['forum_history_month']) && isset($_GET['forum_history_year'])) {
			return $this->getMonthHistory($forum, $_GET['forum_history_month'], $_GET['forum_history_year']);
		}
		
		$history = $forum->getHistory();
		
		// retrieve the month names for quick access
		$month_names = array();
		for ($i=1; $i <= 12; $i++) {
			$month_names[$i] = $lang->getText('Common.Months.'.$i);
		}
		
		$res .= '<h1>'.$lang->getText('Forum.History', array($forum->getForumName())).'</h1>';
		
		foreach ($history as $year => $months) {
			$res .= '<h2>'.$year.'</h2>';
			
			$cols = array();
			$cols[] = $lang->getText('ForumHistory.MonthTitle');
			$cols[] = $lang->getText('ForumHistory.PostsTitle');
			$res .= $theme->tableTop($cols);
			
			foreach ($months as $month => $posts) {
				$history_url = $url->currentUrl(array('forum_id' => $forum->getForumId(), 'forum_history_month' => $month, 'forum_history_year' => $year));
				
				$row = array();
				$row[] = '<a href="'.$history_url.'">'.$month_names[$month].'</a>';
				$row[] = $lang->getText('ForumHistory.Posts', array($posts));
				$res .= $theme->tableRow($row);
			}
			$res .= $theme->tableBottom();
			$res .= '<br />'."\n";
		}
		
		return $res;
	}
	
	private function getMonthHistory(Forum $forum, $month, $year) {
		$res = '';

		$lang = $this->language;
		$url = GFUrl::getUrl();

		$msg_array = $forum->getMonthHistory($month, $year);
		
		if (count($msg_array) > 0){ 
			
			$limit = $this->getThreadLimit();
			$total = count($msg_array);
			$paginator = new Paginator('forum_history', $limit, $total);
			
			$res .= '<h1>'.$lang->getText('ForumHistory.HistoryTitle', array($forum->getForumName(), $lang->getText('Common.Months.'.$month), $year)).'</h1>';
			
			$offset = $paginator->getOffset();
			for ($i=0; $i < $limit; $i++) {
				$idx = $offset+$i;
				if ($idx >= count($msg_array)) break;
				
				$message = $msg_array[$idx];
				$res .= $this->getMessage($message);
				$res .= '<br />'."\n";
			}
			
			$res .= '<div align="center" class="paginator">'.$paginator->getPaginator().'</div><br />';
		} else {
			$res .= $lang->getText('ForumHistory.NoPosts');
		}

		$res .= '<div align="center"><a href="'.$url->currentUrl(array(), array('forum_history_month','forum_history_year')).'">'.$lang->getText('ForumHistory.Back').'</a></div>';
		
		return $res;
	}
	
	public function showAllThreads() {
		$this->show_only_latest = false;
	}
	
	public function showOnlyLatestThreads() {
		$this->show_only_latest = true;
	}
	
	public function handleBrowse(Forum $forum) {
		$url = GFUrl::getUrl();
		$action = (isset($_GET['_forum_action'])) ? $_GET['_forum_action'] : '';

		switch ($action) {
			case 'ForumThreadAdd': {
				return $this->newThreadForm($forum, $url->currentUrl(array('_forum_action' => 'ForumThreadBrowse')));
				break;
			}
			case 'ForumMessageBrowse': {
				$c = new Criteria();
				$c->add(ForumThreadPeer::FORUM_THREAD_ID, $_GET['thread_id']);
				$thread = ForumThreadPeer::doSelectJoinForum($c);
				
				if (count($thread) == 0) {
					return $Lang->getText('Forum.InvalidThreadId');
				} else {
					$thread = $thread[0];
					return $this->getThreadMessages($thread);
				}
				break;
			}
			case 'MessageReply': {
				$message = ForumMessagePeer::retrieveByPk($_GET['message_id']);

				if (is_null($message)) {
					return $Lang->getText('Forum.InvalidForumMessageID');
				} else {
					$thread = $message->getForumThread();
					$forum = $message->getForumThread()->getForum();
					$params['thread'] = $thread;
					$form = $this->messageReplyForm($message,
						GFUrl::getUrl()->currentURL(array('_forum_action' => 'ForumThreadBrowse')));
					if ($this->hasErrors()) {
						$errors = $this->getErrors();
						$params['feedback'] = '* '.implode('<br />* ', $errors);
					}
					
					return $form;
				}
				
				break;
			}
			case 'MessageArchive': {
				return $this->getHistory($_GET['forum_id']);
				break;
			}
			case 'ForumThreadBrowse':
			default: {
				return $this->getThreads($forum);
				break;
			}
		}
	}
	
	public function setReadonly() {
		$this->readonly = true;
	}
}

?>
