<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ForumTemplate.php 4225 2009-12-04 12:40:42Z mpapayannis $
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');

class ForumTemplate extends ProjectTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='forum';
		$Lang = Session::getSession()->getLanguage();

		if (!isset($params['project']) || !is_object($params['project'])) {
			throw new exception($Lang->getText('ForumTemplate.MissingProject'));
		}
		
		if (isset($params['thread'])) {
			$forum = $params['thread']->getForum();
			$params['forum'] = $forum;
		}

		if (isset($params['forum']) && is_object($params['forum'])) {
			$forum = $params['forum'];
			
			if (isset($params['title'])) {
				$params['title']=' '.$Lang->getText('Forum.TabString').' > '.$forum->getForumName().' > '.$params['title'];
			} else {
				throw new exception('Missing title in forum template');
			}
			
			//add a breadcrumb for the thread if we're in one
			if (isset($_REQUEST['thread_id'])) {
				$ft=ForumThreadPeer::retrieveByPk($_REQUEST['thread_id']);
				if (is_object($ft) && $ft->getForumId() == $params['forum']->getForumId()) {
					$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'ForumBrowse','forum','',array('forum_id' => $forum->getForumId(), '_forum_action'=>'ForumMessageBrowse', 'thread_id'=>$_REQUEST['thread_id'])).'">'.$ft->getThreadName().'</a>');
				}
			}
			$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'ForumBrowse','forum','',array('forum_id' => $forum->getForumId())).'">'.$forum->getForumName().'</a>');
		
		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','forum').'">'.$Lang->getText('Forum.TabString').'</a>');
		}
		
		return parent::header($params);
	}

	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
		$tprs = ActionManager::Notify('GET_FORUM_NAV_ELEMENTS',null);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];

		$pine = $this->getPluginNavElements($params);
		$ret = array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		
		$elements = array();
		
		$forums = ForumPeer::retrieveForums('project', $project->getId());
		
		foreach($forums as $forum) {
			if (!$forum->userCanRead()) {
				continue;
			}
			/*if (isset($params['forum'])) {
			$forum = $params['forum'];*/
			$subelements = array();
			
			
			$subelements[]=array('link' => $url->projectUrl($params['project'],'ForumBrowse','forum','',array('forum_id' => $forum->getForumId())), 'description' => $lang->getText('Forum.Browse'));
			if ($loggedin) {
				$subelements[] = array('link' => $url->projectUrl($project, 'ForumBrowse', 'forum', '', array('forum_id'=>$forum->getForumId(), '_forum_action'=>'ForumThreadAdd')),
					'description' => $lang->getText('ForumThread.AddNew'));

				if ($forum->userIsMonitoring($user)) {
					$subelements[] = array('link' => $url->getMonitorUrl(Forum::SECTION, $forum->getPrimaryKey(), null, false),
						'description' => $lang->getText('Forum.StopMonitoring'));
				} else {
					$subelements[] = array('link' => $url->getMonitorUrl(Forum::SECTION, $forum->getPrimaryKey(), null, true),
						'description' => $lang->getText('Forum.Monitor'));
				}

				$subelements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>Forum::SECTION,'ref_id'=>$forum->getForumId(),'description'=>$forum->getForumName(),'type'=>'plugin','return_to'=>$_SERVER['REQUEST_URI'])),
					'description' => $lang->getText('Monitor.UsersMonitoringItem'));

			}
			
	/*		$elements[] = array('link' => $url->currentUrl(array('_forum_action' => 'MessageArchive', 'forum_id' => $forum->getPrimaryKey())),
						'description' => $lang->getText('Forum.ViewArchive'));
	*/					
			// Only show the "save place" option when browsing threads
			if ($loggedin && !isset($_GET['_forum_action'])) {
				$sublements[] = array('link' => $url->currentUrl(array('forum_save' => 1)),
					'description' => $lang->getText('Forum.SavePlace'));						
			}		
			
			if ($loggedin) {
				if ($project->getPermission()->isAdmin() || $project->getPermission()->check(ForumPlugin::SECTION,$forum->getForumId(),ForumPlugin::ADMIN)) {
					$subelements[] = array('link' => $url->projectURL($project,'ForumEdit','forum','admin', array('id'=>$forum->getForumId())),
					'description' => $lang->getText('Tabs.Admin'));
				}
			}
			if(isset($params['forum']) && $params['forum']->getForumId()==$forum->getForumId()) {
				$expanded=true;
			} else {
				$expanded=false;
			}
			
			$elements[] = array('link'=>'',
			'description'=> $forum->getForumName(),
			'expandables' => $subelements,
			'expanded' => $expanded);
		}

		//show admin link for all forums
		if ($loggedin) {
			if ($project->getPermission()->isAdmin() || $project->getPermission()->check(ForumPlugin::SECTION,0,ForumPlugin::ADMIN)) {
				$elements[] = array('link' => $url->projectURL($project,'','forum','admin', array()),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}

		if (count($elements) > 0) {
			$ret[] = array(
				'title' => $lang->getText('Forum.TabString'),
				'elements'=> $elements
			);
		}
		
		return $ret;
	}
	
	function generateRSSLinks() {
		$url = GFUrl::getUrl();
		$project = $url->getProject();

//		if (is_object($project) && $project->isPublic()) {
			// Retrieve the forum objects array
			$crit = new Criteria();
			$crit->add(ForumPeer::REF_ID, $project->getProjectId());
			$crit->add(ForumPeer::SECTION, Project::SECTION);
			$crit->addAscendingOrderByColumn(ForumPeer::FORUM_NAME);
			$arr = ForumPeer::doSelect($crit);
//		} else {
//			$arr = array();
//		}

		$links_arr = parent::generateRSSLinks();

		if(is_array($arr) && count($arr) > 0){
			$i = 0;
			foreach ($arr as $item) {
				if ($item->userCanRead()) {
					$links_arr['forum'.$i]['title'] = GFConf::get('system.name').' - '.$project->getName().' - Forum: '.$item->getForumName();
					$links_arr['forum'.$i]['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => Forum::SECTION, 'ref_id' => $item->getForumID(), 'project_id' => $project->getProjectId())));
					$i++;
				}
			}
		}

		return $links_arr;
	}

}

?>
