<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id: ScmPostCommit.php 3226 2008-08-06 17:58:08Z bigdisk $
 * @author Daniel A. Perez daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


/**
 * This script takes some POST variables. It makes some checks and it
 * stores the commit data in the database.
 * It is used to interact with external SCM scripts.
 */
 
require_once('plugins/scm/lib/SCMRepositoryFactory.php');
require_once('plugins/scm/lib/ScmCommitWrapper.php');

class NewScmCommit {
	private $project;
	
	function __construct($project) {
		$this->project = $project;
		
		if (!isset($_POST['username'])) {
			$this->returnError('User name not set in POST data');
		} elseif (!isset($_POST['log'])) {
			$this->returnError('Log information not set in POST data');
		} elseif (!isset($_POST['serialized_files'])) {
			$this->returnError('File information not set in POST data');
		}
		
		$repos = ScmRepositoryFactory::getRepository($this->project);
		if (is_null($repos)) {
			$this->returnError('Unable to retrieve repository object for project '.$this->project->getUnixName());
		}
		
		$username = $_POST['username'];
		$user = UserPeer::retrieveByUnixName($username);
		$log = $_POST['log'];
		$files = unserialize($_POST['serialized_files']);
		
		try {
			$wrapper = new ScmCommitWrapper($repos, $user, $log, $files);
			$wrapper->generateCommitData();
		} catch (Exception $e) {
			$this->returnError($e->getMessage());
		}
		
		$this->returnSuccess();
	}


	private function returnError($log, $error_code=1) {
		$result = array(
					'error' => $error_code,
					'log' => $log
				);
		echo serialize($result);
		die();
	}
	
	private function returnSuccess($message='') {
		$result = array(
					'error' => 0,
					'log' => $message
				);
		echo serialize($result);
		die();
	}	
}

$project = GFUrl::getUrl()->getProject();
$scmcommit = new NewScmCommit($project);



?>