<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: AccessInfo.php 3226 2008-08-06 17:58:08Z bigdisk $
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFUrl.php');
require_once('plugins/scmgit/lib/html/ScmgitTemplate.php');
require_once('plugins/scmgit/lib/GITRepository.php');


$project = GFUrl::getUrl()->getProject();
$gitplugin = $project->getPluginByName('scmgit');
if (is_null($gitplugin)) {
	throw new InvalidPluginException('scmgit');
}

$session = Session::getSession();
$lang = $session->getLanguage();

$template = new ScmgitTemplate();

$params = array(
	'selectedbottomtab'	=> 'scmgit',
	'project' => $project,
	'action' => isset($action) ? $action : '',
	'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
	'title' => $lang->getText('Scmgit.AccessInfo')
	);

$template->header($params);

// See if SCM is configured for this project
$repos = new GitRepository($project);
if ($repos->isConfigured()) {
	$useSSH = GFConf::get('scmgit.useSSH');
	$projectScm = $repos->getScmConfiguration();

	if (!$projectScm->getControlled()) {
		//@todo	This will be used when we add support for external SCM
		//die("UNSUPPORTED");
	}
	
	$root = GFConf::get('scmgit.localGitRoot');
	if (GFConf::get('system.splitScmServers')) {
		$hostname = $projectScm->getScmServer()->getHostname();
	} else {
		$hostname = GFConf::get('system.hostName');
	}

	$custom_access_text = $projectScm->getAccessText();
	if (!empty($custom_access_text)) {
?>
<p>
	<h3><?php echo $lang->getText('Scmgit.AccessTextTitle') ?></h3>
</p>
<p>
	<?php echo $custom_access_text ?>
</p>
<?php
	}

	if ($projectScm->getControlled()) {		// Only display the access info if access is controlled
		if ($project->getIsPublic() && $projectScm->getEnableAnonread()) {
?>
<p>
	<h3><?php echo $lang->getText('Scmgit.AnonymousAccess'); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmgit.AnonymousAccessDescription'); ?>
</p>
<p>
<?php
	if ($useSSH) {
		echo '<tt>git clone ssh://anonymous@'.$hostname.$repos->getRoot().'</tt><br />'."\n";
	} else {
		//echo '<tt>svn checkout '.GFUrl::getUrlPrefix() . $hostname .'/svn/'.$repos->getRepositoryName().'</tt><br />'."\n";
	}
?>
<br /><br />
</p>
<?php 	} /*if ($projectScm->getEnableAnonread()) */ ?>
<p>
	<h3><?php echo ($useSSH ? $lang->getText('Scmgit.DeveloperAccessSSH') : $lang->getText('Scmgit.DeveloperAccess')); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmgit.DeveloperAccessDescription'); ?>
</p>
<p>
	<?php
		if ($useSSH) {
			echo '<tt>git clone ssh://<i>developername</i>@'.$hostname.$repos->getRoot().'</tt><br />'."\n";
		} else {
			//echo '<tt>svn checkout --username <i>developername</i> '.GFUrl::getUrlPrefix() . $hostname .'/svn/'.$repos->getRepositoryName().'</tt><br />'."\n";
		}
	?>
</p>

<?php
	} /* $projectScm->getControlled() */
} else {		// SCM is not configured for this project
?>
	<a href="<?php echo GFUrl::getUrl()->projectURL($project, '', 'scmgit', 'admin'); ?>"><?php echo $lang->getText('Scm.Admin'); ?></a>
	<p><h3><?php echo $lang->getText('Scm.AdminTitle'); ?></h3></p>
	<p><?php echo $lang->getText('Scm.NotConfigured') ?></p>

<?php
}
$template->footer($params);
?>
