<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ProjectScmEditAction.php 3226 2008-08-06 17:58:08Z bigdisk $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a ProjectScm and saves it to the 
// database
//
require_once('plugins/scm/lib/ProjectScm.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');
require_once('plugins/scm/lib/ScmPreferencePeer.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();

$project = GFUrl::getUrl()->getProject();
$svnplugin = $project->getPluginByName('scmsvn');
if (is_null($svnplugin)) {
	throw new InvalidPluginException('scmsvn');
}

$repos = new SVNRepository($project);
$ProjectScm = $repos->getScmConfiguration();

// $ProjectScm can be null if SCM is not currently configured for this project
if (!is_object($ProjectScm)) {
	if (!$project->getPermission()->isAdmin()) {
		throw new PermissionDeniedException();
	} else {
		// this means it's the first time we're configuring SCM for this project
		$ProjectScm = new ProjectScm();
		$ProjectScm->setProjectId($project->getProjectId());
	}
} else if (!$ProjectScm->userCanUpdate()) {
	throw new PermissionDeniedException();
}
/*
$errors = array();

if (empty($_POST['scm_server_id']) || !is_numeric($_POST['scm_server_id'])) {
	$errors['scm_server_id'] = $lang->getText('ProjectScm.EmptyServerId');
}

if (count($errors) == 0) {
	$ProjectScm->setScmServerId( Validator::sanitizeInt($_POST['scm_server_id']) );
	
	// The following are hardcoded for now, in the future they'll be used to 
	// allow remote SCM
	$ProjectScm->setControlled(true);
	$ProjectScm->setHostname(null);
	$ProjectScm->setRootDirectory(null);
	
	$ProjectScm->save();
	
	// Forces reloading of the object. This is needed if we just created the object.
	ProjectScmPeer::invalidate($project->getProjectId());

	$feedback = $Lang->getText('ProjectScm.ChangesSaved');
} else {
	$feedback = implode('<br />', $errors);
}
*/

$scm_pref = ScmPreferencePeer::retrieveByPk($project->getProjectId());
$scm_pref->setProjectId($project->getProjectId());

try {
	if (isset($_POST['require_tracker_item_id'])) {
		$scm_pref->setRequireTrackerItemId(true);
	} else {
		$scm_pref->setRequireTrackerItemId(false);
	}
	if (isset($_POST['mail_commits'])) {
		$scm_pref->setMailCommits(true);
	} else {
		$scm_pref->setMailCommits(false);
	}
	$scm_pref->save();

	$ProjectScm->setRootDirectory(null);
	
	if (!$ProjectScm->getControlled()) {
		$ProjectScm->setExternalBrowseUrl(Validator::sanitizeString($_POST['external_browse_url']));
	}

	$ProjectScm->setBrowseText( Validator::sanitizeHtml($_POST['browse_text']) );
	$ProjectScm->setAccessText( Validator::sanitizeHtml($_POST['access_text']) );
	$ProjectScm->save();
	
	$feedback = $Lang->getText('ProjectScm.ChangesSaved');
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
	$feedback = '* '.implode('<br />* ', $errors);
}


include('ProjectScmEdit.php');

?>
