<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: index.php 620 2006-03-23 16:13:34Z tperdue $
 * @author Federico Tarantini federico@gforgegroup.com
 * @date 2006-05-24
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFUrl.php');
require_once('plugins/scmvss/lib/html/ScmVSSTemplate.php');
require_once('plugins/scmvss/lib/VSSRepository.php');

$project = GFUrl::getUrl()->getProject();

$vssplugin = $project->getPluginByName('scmvss');
if (is_null($vssplugin)) {
	throw new InvalidPluginException('scmvss');
}

$session = Session::getSession();
$lang = $session->getLanguage();

$template = new ScmvssTemplate();
//prevent SCM spider indexation
$indexSCM = false;
if(GFConf::varExists('system.indexSCM')){
	$indexSCM = GFConf::get('system.indexSCM');
}
if(!$indexSCM){
	$extrahead[] = '<META NAME="ROBOTS" CONTENT="NOINDEX,NOFOLLOW" />';
}
$params = array(
	'selectedbottomtab'	=> 'scmvss',
	'project' => $project,
	'action' => isset($action) ? $action : '',
	'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
	'title' => $lang->getText('Scmvss.InfoTitle'),
	'extra_head' => $extrahead
	);


$template->header($params);

// See if SCM is configured for this project
$repos = new VSSRepository($project);

if ($repos->isConfigured()) {
	//shows VSS repository access info
	$conf = $repos->getScmConfiguration();

	// get VSS servers info
	$c = new Criteria();
	$c->add(ScmServerPeer::SCM_SERVER_ID, $conf->getScmServerId());
	$hostname = $conf->getHostname();
	$path = $conf->getRootDirectory();
	$custom_access_text = $conf->getAccessText();
	$external_browse_url = $conf->getExternalBrowseUrl();
	$browse_text = $conf->getBrowseText();

?>
<p><h3><?php echo $lang->getText('Scmvss.AccessInfo');?></h3></p>
<p>
	<?php
	if (empty($custom_access_text)) {
		echo $lang->getText('Scmvss.AccessInfoDescription');
	} else {
		echo $custom_access_text;
	}
	?>
	<br />
	<?php
		echo '<pre>\\\\'.$hostname;
		if (strpos($path, '\\') !== 0) echo '\\';
		echo $path.'</pre>';
	?>
</p>
<?php
	if (!empty($external_browse_url) || !empty($browse_text)) {
?>
<p><h3><?php echo $lang->getText('Scm.ExternalBrowsing') ?></h3></p>
<p>
	<?php echo $browse_text; ?><br />
	<?php if (!empty($external_browse_url)) echo '<br /><a href="'.$external_browse_url.'">'.$external_browse_url.'</a>'; ?>
</p>
<?php
	}
} else {
	//if admin, it let you fill the access info
	if ($project->getPermission()->isAdmin()) {
?>
		<a href="<?php echo GFUrl::getUrl()->projectURL($project, '', 'scmvss', 'admin'); ?>"><?php echo $lang->getText('Scmvss.Admin'); ?></a>
<?php
	}
?>
	<p><h3><?php echo $lang->getText('Scm.AdminTitle'); ?></h3></p>
	
	<p><?php echo $lang->getText('Scm.NotConfigured') ?></p>
<?php
}

$template->footer($params);
?>
