<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2007-09-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTrackerCommitTrigger.php';
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerCommitTrigger extends BaseTrackerCommitTrigger {
	const ACTION_CREATE = 1;
	const ACTION_UPDATE = 2;
	const ACTION_DELETE = 3;

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TrackerCommitTrigger::setDeleted()
	 * @see TrackerCommitTrigger::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		
		ActionManager::notify("TRACKER_COMMIT_TRIGGER_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (!$this->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		
		$isnew = $this->isNew();
		
		ActionManager::notify("TRACKER_COMMIT_TRIGGER_SAVE_BEFORE", $this);
		parent::save($con);
		ActionManager::notify("TRACKER_COMMIT_TRIGGER_SAVE", $this);
		
		if ($isnew) {
			$this->updateTemplate(self::ACTION_CREATE);
		} else {
			// This shouldn't be triggered!
			$this->updateTemplate(self::ACTION_UPDATE);
		}

	}

	private function updateTemplate($action) {
		// Replicate the data only if it's a template tracker and it's a
		// site admin
		if (!$this->getTracker()->isTemplateTracker() || !Session::getSession()->isSiteAdmin()) {
			return;
		}
		

		$dependingTrackers = $this->getTracker()->getDependingTrackers();
		foreach ($dependingTrackers as $dependingTracker) {
			if ($dependingTracker->getProject()->getProjectId() == $this->getTracker()->getProject()->getProjectId()) {
				// Shouldn't happen, means that the project.template_project_id=project.project_id
				// but we avoid infinite loop anyways
				continue;
			}
			
			$extra_field_to = $this->getTrackerExtraField();
			$element_to = $this->getTrackerExtraFieldElement();
			
			// Get the associated tracker extra field  & item (if it exists)
			$c = new Criteria();
			$c->add(TrackerExtraFieldElementPeer::ELEMENT_NAME, $element_to->getElementName());
			$c->addJoin(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldElementPeer::TRACKER_EXTRA_FIELD_ID);
			$c->add(TrackerExtraFieldPeer::FIELD_NAME, $extra_field_to->getFieldName());
			$c->add(TrackerExtraFieldPeer::TRACKER_ID, $dependingTracker->getTrackerId());
			
			$depending_element_to = TrackerExtraFieldElementPeer::doSelectOne($c);
			if (!$depending_element_to) {
				// Matching element not found
				continue;
			}
			$depending_extra_field_to = $depending_element_to->getTrackerExtraField();
			
			$c = new Criteria();
			$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $dependingTracker->getTrackerId());
			// For now, only one trigger is possible
			//$c->add(TrackerCommitTriggerPeer::CHANGE_TRACKER_EXTRA_FIELD_ID, $depending_extra_field_to->getTrackerExtraFieldId());
			//$c->add(TrackerCommitTriggerPeer::TO_ELEMENT_ID, $depending_element_to->getElementId());
			$existing_trigger = TrackerCommitTriggerPeer::doSelectOne($c);

			switch ($action) {
			case self::ACTION_CREATE:
				if ($existing_trigger) {
					$existing_trigger->delete();
				}

				$trigger = new TrackerCommitTrigger();
				$trigger->setTrackerId($dependingTracker->getTrackerId());
				$trigger->setChangeTrackerExtraFieldId($depending_extra_field_to->getTrackerExtraFieldId());
				$trigger->setToElementId($depending_element_to->getElementId());
				$trigger->save();
				continue;
			case self::ACTION_UPDATE:
				//unimplemented
				continue;
			case self::ACTION_DELETE:
				if ($existing_trigger) {
					$existing_trigger->delete();
				}
				continue;
			}
		}
	}


}
?>
