<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

class TrackerItemQuery {
	const FIELD_ASSIGNEE='1';
	const FIELD_MODDATE='3';
	const FIELD_EXTRAFIELD='4';
	const FIELD_SORTCOL='5';
	const FIELD_SORTORD='6';
	const FIELD_OPENDATE='7';
	const FIELD_CLOSEDATE='8';
	const FIELD_LIST='9';
	const FIELD_PRIORITY='10';

	protected $tracker_item_id;
	protected $summary;
	protected $assignee;
	protected $extra_fields;
	protected $moddaterange;
	protected $opendaterange;
	protected $closedaterange;
	protected $sortord;
	protected $sortcol;
	protected $TrackerItems;
	protected $priority;
	protected $field_list;
	protected $Tracker;
	protected $showcols;

	function __construct(Tracker $Tracker) {
		$this->Tracker = $Tracker;
		$this->setupShowColumns();
	}

	function getTrackerItemId(){
		return $this->tracker_item_id;
	}
	function getSummary() {
		return $this->summary;
	}
	function getAssignedTo() {
//echo "Assignee:".$this->assignee;
		return $this->assignee;
	}
	function getExtraFields() {
		return $this->extra_fields;
	}
	function getModDateRange() {
		if ($this->moddaterange)
			return $this->moddaterange;
	}
	function getOpenDateRange() {
		if ($this->opendaterange)
			return $this->opendaterange;
	}
	function getCloseDateRange() {
		if ($this->closedaterange)
			return $this->closedaterange;
	}
	function getSortOrd() {
//echo "Sort Ord".$this->sortord;
		if (isset($this->sortord) && $this->sortord=='DESC') {
			return 'DESC';
		} else {
			return 'ASC';
		}
	}
	function getSortCol() {
//echo "Sort Col".$this->sortcol;
		if (isset($this->sortcol)) {
			$arr = $this->getAvailableSortColumns();
			if (isset($arr[$this->sortcol])) {
				return $this->sortcol;
			} else {
				return 'tracker_item_id';
			}
		} else {
			return 'tracker_item_id';
		}
	}
	function getPriority() {
//echo "priority".$this->priority;
		return $this->priority;
	}

	function getWhereClause() {
		//this may have been called already
		if (isset($this->TrackerItems) && is_array($this->TrackerItems)) {
			return $this->TrackerItems;
		}

		if (isset($this->summary) && strlen($this->summary) > 0) {
			$summary_str=' AND summary ILIKE \'%'.$this->summary.'%\' ';
		} else {
			$summary_str='';
		}
		//if assigned to selected, and more to where clause
		if (isset($this->assignee) && $this->assignee > 0) {
			if (is_array($this->assignee)) {
				if  (count($this->assignee) > 0) {
					$assigned_str="AND tia.tracker_item_id=ti.tracker_item_id AND tia.assignee IN ('".implode("','",$this->assignee)."')";
					$assigned_table_str=', tracker_item_assignee tia';
				} else {
					$assigned_str='';
					$assigned_table_str='';
				}
			} else {
				$assigned_str="AND tia.tracker_item_id=ti.tracker_item_id AND tia.assignee='".$this->assignee."'";
				$assigned_table_str=', tracker_item_assignee tia';
			}
		} else {
			//no assigned to was chosen, so don't add it to where clause
			$assigned_table_str='';
			$assigned_str='';
		}

		if (isset($this->priority) && $this->priority > 0) {
			if (is_array($this->priority)) { 
				if (count($this->priority) > 0) {
					$priority_str=" and ti.priority IN ('".implode("','",$this->priority)."') ";
				} else {
					$priority_str='';
				}
			} else {
				$priority_str=" and ti.priority='".$this->priority."' ";
			}
		} else {
			$priority_str='';
		}

		$ef_table_str='';
		$ef_where_str='';
		if (isset($this->extra_fields) && is_array($this->extra_fields)) {
			$keys=array_keys($this->extra_fields);
			$vals=array_values($this->extra_fields);
			for ($i=0; $i<count($keys); $i++) {
				if (empty($vals[$i])) {
					continue;
				}
				$tef=TrackerExtraFieldPeer::retrieveByPk($keys[$i]);
				if (!is_object($tef) || $tef->getTrackerId() != $this->Tracker->getTrackerId()) {
					continue;
				}
//date types have to be cast and use BETWEEN
				if ($tef->getFieldType()==TrackerExtraField::TYPE_DATE) {

					$ef_table_str.=", tracker_extra_field_data aefd$i ";
					$ef_where_str.=" AND aefd$i.tracker_extra_field_id='".$keys[$i]."'
						AND aefd$i.field_data != '' AND aefd$i.field_data::date BETWEEN '".$vals[$i][0]."' AND '".$vals[$i][1]."'
						AND aefd$i.tracker_item_id=ti.tracker_item_id ";
//NUMERIC and SERIAL types have to use BETWEEN
				} elseif ($tef->getFieldType()  == TrackerExtraField::TYPE_NUMERIC || $tef->getFieldType() == TrackerExtraField::TYPE_SERIAL) {

						$ef_table_str.=", tracker_extra_field_data aefd$i ";
						$ef_where_str.=" AND aefd$i.tracker_extra_field_id='".$keys[$i]."'
						AND aefd$i.field_data != '' AND aefd$i.field_data::float BETWEEN ".$vals[$i][0]." AND ".$vals[$i][1]."
						AND aefd$i.tracker_item_id=ti.tracker_item_id ";

				} else {

					if (is_array($vals[$i]) && !empty($vals[$i])) {
						$vals[$i]=implode("','",$vals[$i]);
					}
					$ef_table_str.=", tracker_extra_field_data aefd$i ";
					$ef_where_str.=" AND aefd$i.tracker_extra_field_id='".$keys[$i]."'
						AND aefd$i.field_data IN ('".$vals[$i]."') 
						AND aefd$i.tracker_item_id=ti.tracker_item_id ";

				}
			}
		}

		//add constraint of range of modified dates
		if (isset($this->moddaterange)) {
			$range_arr=explode(' ',$this->moddaterange);
			$begin = $range_arr[2] . '-' . $range_arr[0] . '-' . $range_arr[1];
			$end = $range_arr[5] . '-' . $range_arr[3] . '-' . $range_arr[4];
			$moddatesql= " AND ti.last_modified_date BETWEEN '$begin' AND '$end' ";
		} else {
			$moddatesql= '';
		}
		//add constraint of range of open dates
		if (isset($this->opendaterange)) {
			$range_arr=explode(' ',$this->opendaterange);
			$begin = $range_arr[2] . '-' . $range_arr[0] . '-' . $range_arr[1];
			$end = $range_arr[5] . '-' . $range_arr[3] . '-' . $range_arr[4];
			$opendatesql= " AND ti.open_date BETWEEN '$begin' AND '$end' ";
		} else {
			$opendatesql= '';
		}
		//add constraint of range of close dates
		if (isset($this->closedaterange)) {
			$range_arr=explode(' ',$this->closedaterange);
			$begin = $range_arr[2] . '-' . $range_arr[0] . '-' . $range_arr[1];
			$end = $range_arr[5] . '-' . $range_arr[3] . '-' . $range_arr[4];
			$closedatesql= " AND ti.close_date BETWEEN '$begin' AND '$end' ";
		} else {
			$closedatesql= '';
		}
		//add constraint for restricted browing
		if ($this->Tracker->getRestrictBrowse() && !$this->Tracker->isPublic()) {
			if (!$this->Tracker->getProject()->getPermission()->check('tracker',$this->Tracker->getTrackerId(),Tracker::PERMLEVEL_TECH)) {
				if (!Session::getSession()->isLoggedIn()) {
					Throw new PermissionDeniedException();
				} else {
					$restrict_str=" AND submitted_by='".Session::getSession()->getUser()->getUserId()."'";
				}
			} else {
				$restrict_str='';
			}
		} else {
			$restrict_str='';
		}

		$qstr="$ef_table_str $assigned_table_str
			WHERE
			ti.tracker_id='". $this->Tracker->getTrackerID() ."'
			$opendatesql $moddatesql $closedatesql $priority_str $summary_str
			 $assigned_str $ef_where_str $restrict_str ";
		if ($this->getTrackerItemId() > 0) {
			$qstr=" WHERE
			ti.tracker_id='". $this->Tracker->getTrackerID() ."'
			AND ti.tracker_item_id='".$this->getTrackerItemId()."'
			$restrict_str ";
		}
		return $qstr;
	}

	function getTrackerItemCount() {
		
		$sql="SELECT count(*) FROM tracker_item ti ".$this->getWhereClause();
//echo $sql;

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		while ($rs->next()) {
			return $rs->getInt(1);
		}
	}

	function getTrackerItems($limit=0, $offset=0) {
		//
		// add LIMIT if necessary
		//
		
		$limit = Validator::sanitizeInt($limit);
		$offset = Validator::sanitizeInt($offset);
		
		$limitStr="";
		if ($limit > 0 && $offset > 0) {
//PGSQL POSTGRESQL specific
			$limitStr = "LIMIT $limit OFFSET $offset";
		}
		else if ($limit > 0 && $offset == 0) {
			$limitStr = "LIMIT $limit";
		}
		else if ($limit == 0 && $offset > 0) {
			$limitStr = "LIMIT ALL, $offset";
		}

		//
		//  now run the query using the criteria chosen above
		//
		//$sql="SELECT * FROM tracker_item WHERE tracker_item_id IN (
		$sql="SELECT * FROM tracker_item WHERE tracker_item.tracker_item_id IN (SELECT ti.tracker_item_id as ti_id FROM tracker_item ti ".$this->getWhereClause().") ORDER BY tracker_item.".$this->getSortCol()." ".$this->getSortOrd()." $limitStr";
		//$sqlcount="SELECT count(*) FROM tracker_item ti $qstr";
//echo $sql;

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$this->TrackerItems = TrackerItemPeer::populateObjects($rs);
		return $this->TrackerItems;
	}

	function getAvailableColumns() {
		$Lang=Session::getSession()->getLanguage();
		$cols=array();
		$cols['priority']=$Lang->getText('TrackerItem.PriorityName');
		$cols['assigned_to']=$Lang->getText('TrackerItem.AssignedTo');
		$cols['submitted_by']=$Lang->getText('TrackerItem.SubmittedByName');
		$cols['open_date']=$Lang->getText('TrackerItem.OpenDateName');
		$cols['close_date']=$Lang->getText('TrackerItem.CloseDateName');
		$cols['last_modified_date']=$Lang->getText('TrackerItem.LastModifiedDateName');
		$efs = $this->Tracker->getExtraFields();
		foreach ($efs as $ef) {
			$cols[$ef->getTrackerExtraFieldId()]=$ef->getFieldName();
		}
		return $cols;
	}

	function getAvailableSortColumns() {
		$Lang=Session::getSession()->getLanguage();
		$cols=array();
		$cols['tracker_item_id'] = $Lang->getText('TrackerItem.TrackerItemIdName');
		$cols['summary'] = $Lang->getText('TrackerItem.SummaryName');
		$cols['priority']=$Lang->getText('TrackerItem.PriorityName');
		$cols['submitted_by']=$Lang->getText('TrackerItem.SubmittedByName');
		$cols['open_date']=$Lang->getText('TrackerItem.OpenDateName');
		$cols['close_date']=$Lang->getText('TrackerItem.CloseDateName');
		$cols['last_modified_date']=$Lang->getText('TrackerItem.LastModifiedDateName');
		return $cols;
	}

	function getAvailableSortOrd() {
		$Lang=Session::getSession()->getLanguage();
		$sort=array();
		$sort['ASC']=$Lang->getText('Tracker.SortAsc');
		$sort['DESC']=$Lang->getText('Tracker.SortDesc');
		return $sort;
	}

	function setupShowColumns() {
		$cols=array();
		$Lang=Session::getSession()->getLanguage();
		$cols['priority']=$Lang->getText('TrackerItem.PriorityName');
		$cols['assigned_to']=$Lang->getText('TrackerItem.AssignedTo');
		$cols['submitted_by']=$Lang->getText('TrackerItem.SubmittedByName');
		$cols['open_date']=$Lang->getText('TrackerItem.OpenDateName');
		$cols['close_date']=$Lang->getText('TrackerItem.CloseDateName');
		$cols['last_modified_date']=$Lang->getText('TrackerItem.LastModifiedDateName');
		$efs = $this->Tracker->getTrackerExtraFields();
		foreach ($efs as $ef) {
			if ($ef->getUseInQuickBrowse()) {
				$cols[$ef->getTrackerExtraFieldId()]=$ef->getFieldName();
			}
		}
		$this->showcols=$cols;
	}
	function showColumn($column) {
		return isset($this->showcols[$column]);
	}
}

?>
