<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-13
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once ("lib/external/jpgraph/jpgraph.php");
require_once ("lib/external/jpgraph/jpgraph_bar.php");

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();

$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
$stmt = $con->createStatement();

// Check if we want to filter by user
$user = (isset($_GET['user_id']) ? Validator::sanitizeInt($_GET['user_id']) : 0);
$begin = Validator::sanitizeDate($_GET['start_date']);
$end = Validator::sanitizeDate($_GET['end_date']);
$start = date('Y-m-d', (strtotime($end)-1.5*365*60*60*24));
$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

$sqltime = "AND date_entry BETWEEN '$start' AND '$end'";

// Get all valid Project Ids
$all_ids = array();
if ($subprojects == 1) {
	$all_ids = $Project->getAllSubprojectIds();
	$all_ids[] = $project_id;
} else {
	$all_ids[] = $project_id;
}

// If there's a valid project_id, we'll filter by project all the queries
if ($user > 0) {
	$sqluser = "AND tit.user_id = '$user'";
} else {
	$sqluser = '';
}

$sql = "SELECT date_part('year', tit.date_entry)::text || '-' || date_part('month', tit.date_entry)::text AS month,
sum(tit.hours) AS hours
FROM tracker_item_time tit, tracker_item ti, tracker t
WHERE tit.tracker_item_id = ti.tracker_item_id
AND ti.tracker_id = t.tracker_id
AND t.project_id IN (".implode(',',$all_ids).") 
$sqltime
$sqluser
GROUP BY month ORDER BY month";


$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

//Build an array of the last 18 months in YYYY-MM format
$months=array();
$startdate=(strtotime($end)-1.5*365*60*60*24);
$addmonth=intval(date('m',$startdate));
$startyear=intval(date('Y',$startdate));
$addyear=0;
$months["".($startyear+$addyear).'-'.($addmonth).""]=0;
for ($i=0; $i<18; $i++) {
	$addmonth++;
	if ($addmonth > 12) {
		$addmonth=1;
		$addyear++;
	}
	$months["".($startyear+$addyear).'-'.($addmonth).""]=0;
}

$data=array();
$lastYear=0;
$i = -1;
if ($rs != null && $rs->getRecordCount() > 0) {
	while($rs->next()) {
		if (isset($months[$rs->getString('month')])) {
			$months[$rs->getString('month')] = $rs->getString('hours');
		}
	}
}

$font = $Lang->getFontProperties(FF_FONT1, FS_NORMAL, 12);
// Create the graph. These two calls are always required
$graph = new Graph(400, 250);
$graph->title->Set($Lang->getText('TimeTrackingReporting.ByMonth'));
$graph->title->SetFont($font[0], $font[1], $font[2]);
$graph->SetScale('textlin');
$graph->setBox(false);
$graph->setFrame(false);
$graph->img->SetMargin(40,20,30,50);

$font = $Lang->getFontProperties(FF_FONT1, FS_BOLD, 8);
$graph->yaxis->scale->SetGrace(0);
$graph->yaxis->SetLabelMargin(1);
$graph->yaxis->SetTitle($Lang->getText('TimeTrackingReporting.Hours'), 'low');
$graph->yaxis->title->SetFont($font[0], $font[1], $font[2]);
$graph->yaxis->SetTitleMargin(30);
$graph->xaxis->SetTickLabels(array_keys($months));
$graph->xaxis->SetLabelAngle(90);

$b = new BarPlot(array_values($months));
$b->value->Show();
$b->value->SetFormat('%d');
$b->SetAbsWidth(15);
$b->SetWidth(1);

$graph->Add($b);
$graph->Stroke();
?>
 
