<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�ez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/plugin/GFProjectPlugin.php');
require_once('lib/core/UserPluginPeer.php');
require_once('plugin/ActionManager.php');
require_once('SystemUtils.php');
require_once('lib/html/GFWidget.php');

class UserwikiPlugin implements GFProjectPlugin {

	protected $project;

	CONST NOACCESS = 0;
	CONST READ = 1;
	CONST EDIT = 2;
	CONST ADMIN = 3;
	CONST NOANON = 0;
	CONST ANON = 1;
	CONST ISPRIVATE = 0;
	CONST ISPUBLIC = 1;	
	CONST SECTION = "userwiki";
	
	function __construct(/*$project*/) {
		//$this->project = $project;
	}
	

	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	function initialize() {
		$listenTo = array(
			'BUILD_MYPAGE_TABS',
			'BUILD_USER_TABS',
			'GET_WIKI_NAV_ELEMENTS',
			'BUILD_MYPAGE_ELEMENTS'
			);
		ActionManager::registerListeners($this, $listenTo);
	}

	private function _buildMainTabsListener($params) {
		if (!isset($params['user']) || !is_object($params['user']) || !$params['user']->usesPlugin($this->getName())) {
			return;
		}		
		$user = $params['user'];

		$result = new PluginResult($this);
		$components = GFUrl::getUrl()->getURI();
		$arr = explode('/',$components);
		if (in_array('my',$arr)) {
			// for "my" page
			$value = array(
					'text'	=> Language::getSessText('UserWiki.TabString'),
					'action'	=> '/gf/user/'.$user->getUnixName().'/userwiki/my',
					'id'	=> 'userwiki'
				);			
		} else {
			// for "user" page
			$value = array(
					'text'	=> Language::getSessText('UserWiki.TabString'),
					'action'	=> '/gf/user/'.$user->getUnixName().'/userwiki/',
					'id'	=> 'userwiki'
				);				
		}

		$result->setResult($value);
		return $result;
	}

	/**
	 *      getWikiNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getWikiNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Wiki Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Wiki Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	function getType() {
		return GFPlugin::USER_PLUGIN;
	}

	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	function getName() {
		return 'userwiki';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	function getListeners() {
		return array(); //XXX
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed

	 *	@return PluginResult
	 */
	function handleAction($listenerName,$params) {
		switch ($listenerName) {
			case 'BUILD_MYPAGE_TABS':
			case 'BUILD_USER_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'GET_WIKI_NAV_ELEMENTS': {
				return $this->getForumNavElements();
				break;
			}
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
		}
	}

	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	

	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','wiki');
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	function search(GFSearch $search) {
		return NULL;	// XXX
	}

	/**
	 * 	The addMember listener - this hook will fire
	 *	whenever a member is added to a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function addMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The removeMember listener - this hook will fire
	 *	whenever a member is removed from a project.
	 *	@param	User	User object
	 *	@return	boolean
	 */
	function removeMemberListener($params) {
		return false; // XXX
	}

	/**
	 * 	The approveProject listener - this hook will fire
	 *	when a project is approved.
	 *	@param	Project	the project object
	 *	@return	boolean
	 */
	function approveProjectListener($params) {
		return false; // XXX
	}

	/**
	 * 	The deleteProject listener - this hook will fire
	 *	when a project is deleted.
	 *	@return	boolean
	 */
	function deleteProjectListener() {
		return false; // XXX
	}

	/**
	 * 	The checkPermission - this listener will fire
	 *	when permissions are being checked for this project.
	 *	@param	Array	An associative array containing 
	 *	'section_name' a name like 'tracker' or 'tracker_item'
	 *	'section_id'=>id (the id of the section, for example a specific tracker)
	 *	'subsection_id'=>id (the id of the subsection, for example an tracker_item id)
	 *	'perm_level'=>int (the value, such a 1 for read, 2 for write, that is being requested)
	 *	@return	boolean
	 * /
	function checkPermission($params);
*/

	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */	
	function getAdminRoleSettings() {
		$return = array();
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings or null
	 *	@return	array
	 */
	function getRoleChoices($params) {

		$return = array();
		return $return;
	}

	/**
	 * returns the settings for the wiki posting
	 *
	 * @return array
	 */
	function getPostingObserverSettings() {
		$return = array();
		return $return;		
	}

	/**
	 * returns the settings for the wiki status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		return $return;		
	}	
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {
		$return = array();
		return $return;		
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($data) {

	}
	
	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			default : {
				return '';
			}
		}
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		
	}		
	
	public function enableInUser() {
		$user = Session::getSession()->getUser();
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME,$this->getName());
		$plugin = PluginPeer::doSelectOne($c);
		$up = new UserPlugin();
		if (($user instanceof User) && ($plugin instanceof Plugin)) {
			$up->setUser($user);
			$up->setPlugin($plugin);
			try {
				$up->save();
			} catch (PropelException $e) {
				// just trying to double insert
				return;
			}
		}
	}
	
	public function disableFromUser() {
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME,$this->getName());
		$plugin = PluginPeer::doSelectOne($c);
		$c->clear();
		$c->add(UserPluginPeer::USER_ID,Session::getSession()->getUser()->getUserId());
		$c->add(UserPluginPeer::PLUGIN_ID,$plugin->getPluginId());
		$pp = UserPluginPeer::doSelectOne($c);
		if (($pp instanceof UserPlugin )) {
			$pp->delete();
		}		
	}
	
	
}

?>
