<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CruisecontrolBuild

require_once('plugins/build/lib/CruisecontrolBuild.php');
require_once('plugins/build/lib/CruisecontrolBuildPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CruisecontrolBuild',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_build_id' => array('name'=>'cruisecontrol_build_id', 'type' => 'xsd:int'),
		'cruisecontrol_id' => array('name'=>'cruisecontrol_id', 'type' => 'xsd:int'),
		'cruisecontrol_defect_id' => array('name'=>'cruisecontrol_defect_id', 'type' => 'xsd:int'),
		'build_date' => array('name'=>'build_date', 'type' => 'xsd:string'),
		'was_successful' => array('name'=>'was_successful', 'type' => 'xsd:boolean'),
		'was_fix' => array('name'=>'was_fix', 'type' => 'xsd:boolean'),
		'was_necessary' => array('name'=>'was_necessary', 'type' => 'xsd:boolean'),
		'description' => array('name'=>'description', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrolBuild',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CruisecontrolBuild[]')),
	'tns:CruisecontrolBuild'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrolBuilds',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
	), 
	array('getCruisecontrolBuilds' => 'tns:ArrayOfCruisecontrolBuild'),
	$uri, 
	$uri.'#getCruisecontrolBuilds',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of CruisecontrolBuild
 * @param	$cruisecontrol_id	Filter by cruisecontrol_id
 */
function getCruisecontrolBuilds($sessionHash, $cruisecontrol_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();
	
	if (!is_null($cruisecontrol_id)) {
		$crit->add(CruisecontrolBuildPeer::CRUISECONTROL_ID, $cruisecontrol_id);
	}
	
	$objs = CruisecontrolBuildPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = CruisecontrolBuildToSOAP($obj);
	}
	}
	return $res;
}

$server->register(
	'getCruisecontrolBuildArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'ArrayOfxsd:int',
	), 
	array('getCruisecontrolBuild' => 'tns:ArrayOfCruisecontrolBuild'),
	$uri, 
	$uri.'#getCruisecontrolBuild',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of CruisecontrolBuild
 * @param	$cruisecontrol_build_id	Retrieve the CruisecontrolBuild array identified by the array of cruisecontrol_build_id
 */
function getCruisecontrolBuildArray($sessionHash, $cruisecontrol_build_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(CruisecontrolBuildPeer::CRUISECONTROL_BUILD_ID,$cruisecontrol_build_idArr,Criteria::IN);
	$objs = CruisecontrolBuildPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = CruisecontrolBuildToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getCruisecontrolBuild',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
	), 
	array('getCruisecontrolBuild' => 'tns:CruisecontrolBuild'),
	$uri, 
	$uri.'#getCruisecontrolBuild',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single CruisecontrolBuild
 * @param	$cruisecontrol_build_id	Retrieve the CruisecontrolBuild identified by this cruisecontrol_build_id
 */
function getCruisecontrolBuild($sessionHash, $cruisecontrol_build_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolBuildPeer::retrieveByPK($cruisecontrol_build_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getCruisecontrolBuild','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return CruisecontrolBuildToSOAP($obj);
}


$server->register(
	'deleteCruisecontrolBuild',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
	), 
	array('deleteCruisecontrolBuild' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrolBuild',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single CruisecontrolBuild
 * @param	$cruisecontrol_build_id	Delete the CruisecontrolBuild identified by this cruisecontrol_build_id
 */
function deleteCruisecontrolBuild($sessionHash, $cruisecontrol_build_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolBuildPeer::retrieveByPK($cruisecontrol_build_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrolBuild','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrolBuild','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateCruisecontrolBuild',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_build_id' => 'xsd:int',
		'cruisecontrol_id' => 'xsd:int',
		'cruisecontrol_defect_id' => 'xsd:int',
		'build_date' => 'xsd:string',
		'was_successful' => 'xsd:boolean',
		'was_fix' => 'xsd:boolean',
		'was_necessary' => 'xsd:boolean',
		'description' => 'xsd:string',
	), 
	array('updateCruisecontrolBuild' => 'xsd:int'),
	$uri, 
	$uri.'#updateCruisecontrolBuild',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update CruisecontrolBuild
 * @param	$cruisecontrol_build_id	 CruisecontrolBuild to update
 * @param	$cruisecontrol_id
 * @param	$cruisecontrol_defect_id
 * @param	$build_date
 * @param	$was_successful
 * @param	$was_fix
 * @param	$was_necessary
 * @param	$description
 */
function updateCruisecontrolBuild($sessionHash, $cruisecontrol_build_id, $cruisecontrol_id, $cruisecontrol_defect_id, $build_date, $was_successful, $was_fix, $was_necessary, $description) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolBuildPeer::retrieveByPK($cruisecontrol_build_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateCruisecontrolBuild', 'Couldn\'t update CruisecontrolBuild: invalid cruisecontrol_build_id '.$cruisecontrol_build_id);
	}
	
	if (!is_null($cruisecontrol_id)) {
		$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	}
	if (!is_null($cruisecontrol_defect_id)) {
		$obj->setCruisecontrolDefectId( Validator::sanitizeInt($cruisecontrol_defect_id) );
	}
	if (!is_null($build_date)) {
		$obj->setBuildDate( Validator::sanitizeDate($build_date) );
	}
	if (!is_null($was_successful)) {
		$obj->setWasSuccessful( Validator::sanitizeBool($was_successful) );
	}
	if (!is_null($was_fix)) {
		$obj->setWasFix( Validator::sanitizeBool($was_fix) );
	}
	if (!is_null($was_necessary)) {
		$obj->setWasNecessary( Validator::sanitizeBool($was_necessary) );
	}
	if (!is_null($description)) {
		$obj->setDescription( Validator::sanitizeString($description) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateCruisecontrolBuild','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addCruisecontrolBuild',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'cruisecontrol_defect_id' => 'xsd:int',
		'build_date' => 'xsd:string',
		'was_successful' => 'xsd:boolean',
		'was_fix' => 'xsd:boolean',
		'was_necessary' => 'xsd:boolean',
		'description' => 'xsd:string',
	), 
	array('addCruisecontrolBuild' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrolBuild',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a CruisecontrolBuild
 * @param	$cruisecontrol_id
 * @param	$cruisecontrol_defect_id
 * @param	$build_date
 * @param	$was_successful
 * @param	$was_fix
 * @param	$was_necessary
 * @param	$description
 */
function addCruisecontrolBuild($sessionHash, $cruisecontrol_id, $cruisecontrol_defect_id, $build_date, $was_successful, $was_fix, $was_necessary, $description) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new CruisecontrolBuild();
	
	$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	$obj->setCruisecontrolDefectId( Validator::sanitizeInt($cruisecontrol_defect_id) );
	$obj->setBuildDate( Validator::sanitizeDate($build_date) );
	$obj->setWasSuccessful( Validator::sanitizeBool($was_successful) );
	$obj->setWasFix( Validator::sanitizeBool($was_fix) );
	$obj->setWasNecessary( Validator::sanitizeBool($was_necessary) );
	$obj->setDescription( Validator::sanitizeString($description) );
	try {
		$obj->save();
		return $obj->getPrimaryKey();
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrolBuild','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function CruisecontrolBuildToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_build_id'] = $obj->getCruisecontrolBuildId();
	$ret['cruisecontrol_id'] = $obj->getCruisecontrolId();
	$ret['cruisecontrol_defect_id'] = $obj->getCruisecontrolDefectId();
	$ret['build_date'] = $obj->getBuildDate();
	$ret['was_successful'] = $obj->getWasSuccessful();
	$ret['was_fix'] = $obj->getWasFix();
	$ret['was_necessary'] = $obj->getWasNecessary();
	$ret['description'] = $obj->getDescription();
	return $ret;
}
?>
