<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Cruisecontrol

require_once('plugins/build/lib/Cruisecontrol.php');
require_once('plugins/build/lib/CruisecontrolPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'Cruisecontrol',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_id' => array('name'=>'cruisecontrol_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'build_name' => array('name'=>'build_name', 'type' => 'xsd:string'),
		'build_description' => array('name'=>'build_description', 'type' => 'xsd:string'),
		'submitter_id' => array('name'=>'submitter_id', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrol',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Cruisecontrol[]')),
	'tns:Cruisecontrol'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrols',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
	), 
	array('getCruisecontrols' => 'tns:ArrayOfCruisecontrol'),
	$uri, 
	$uri.'#getCruisecontrols',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of Cruisecontrol
 * @param	$project_id	Filter by project_id
 */
function getCruisecontrols($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();
	
	if (!is_null($project_id)) {
		$crit->add(CruisecontrolPeer::PROJECT_ID, $project_id);
	}

	$objs = CruisecontrolPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = CruisecontrolToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getCruisecontrolArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'ArrayOfxsd:int',
	), 
	array('getCruisecontrol' => 'tns:ArrayOfCruisecontrol'),
	$uri, 
	$uri.'#getCruisecontrol',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Cruisecontrol
 * @param	$cruisecontrol_id	Retrieve the Cruisecontrol array identified by the array of cruisecontrol_id
 */
function getCruisecontrolArray($sessionHash, $cruisecontrol_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(CruisecontrolPeer::CRUISECONTROL_ID,$cruisecontrol_idArr,Criteria::IN);
	$objs = CruisecontrolPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = CruisecontrolToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getCruisecontrol',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
	), 
	array('getCruisecontrol' => 'tns:Cruisecontrol'),
	$uri, 
	$uri.'#getCruisecontrol',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Cruisecontrol
 * @param	$cruisecontrol_id	Retrieve the Cruisecontrol identified by this cruisecontrol_id
 */
function getCruisecontrol($sessionHash, $cruisecontrol_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getCruisecontrol','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return CruisecontrolToSOAP($obj);
}


$server->register(
	'deleteCruisecontrol',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
	), 
	array('deleteCruisecontrol' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrol',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single Cruisecontrol
 * @param	$cruisecontrol_id	Delete the Cruisecontrol identified by this cruisecontrol_id
 */
function deleteCruisecontrol($sessionHash, $cruisecontrol_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrol','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrol','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateCruisecontrol',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'project_id' => 'xsd:int',
		'build_name' => 'xsd:string',
		'build_description' => 'xsd:string',
		'submitter_id' => 'xsd:int',
	), 
	array('updateCruisecontrol' => 'xsd:int'),
	$uri, 
	$uri.'#updateCruisecontrol',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update Cruisecontrol
 * @param	$cruisecontrol_id	 Cruisecontrol to update
 * @param	$project_id
 * @param	$build_name
 * @param	$build_description
 * @param	$submitter_id
 */
function updateCruisecontrol($sessionHash, $cruisecontrol_id, $project_id, $build_name, $build_description, $submitter_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = CruisecontrolPeer::retrieveByPK($cruisecontrol_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateCruisecontrol', 'Couldn\'t update Cruisecontrol: invalid cruisecontrol_id '.$cruisecontrol_id);
	}
	
	if (!is_null($project_id)) {
		$obj->setProjectId( Validator::sanitizeInt($project_id) );
	}
	if (!is_null($build_name)) {
		$obj->setBuildName( Validator::sanitizeString($build_name) );
	}
	if (!is_null($build_description)) {
		$obj->setBuildDescription( Validator::sanitizeString($build_description) );
	}
	if (!is_null($submitter_id)) {
		$obj->setSubmitterId( Validator::sanitizeInt($submitter_id) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateCruisecontrol','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addCruisecontrol',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'build_name' => 'xsd:string',
		'build_description' => 'xsd:string',
		'submitter_id' => 'xsd:int',
	), 
	array('addCruisecontrol' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrol',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a Cruisecontrol
 * @param	$project_id
 * @param	$build_name
 * @param	$build_description
 * @param	$submitter_id
 */
function addCruisecontrol($sessionHash, $project_id, $build_name, $build_description, $submitter_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new Cruisecontrol();
	
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setBuildName( Validator::sanitizeString($build_name) );
	$obj->setBuildDescription( Validator::sanitizeString($build_description) );
	$obj->setSubmitterId( Validator::sanitizeInt($submitter_id) );
	try {
		$obj->save();
		return $obj->getPrimaryKey();
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrol','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function CruisecontrolToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_id'] = $obj->getCruisecontrolId();
	$ret['project_id'] = $obj->getProjectId();
	$ret['build_name'] = $obj->getBuildName();
	$ret['build_description'] = $obj->getBuildDescription();
	$ret['submitter_id'] = $obj->getSubmitterId();
	return $ret;
}
?>
