<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Forum

require_once('plugins/forum/lib/Forum.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'Forum',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'forum_id' => array('name'=>'forum_id', 'type' => 'xsd:int', 'nillable' => 'false'),
		'section' => array('name'=>'section', 'type' => 'xsd:string', 'nillable' => 'false'),
		'ref_id' => array('name'=>'ref_id', 'type' => 'xsd:int', 'nillable' => 'false'),
		'forum_name' => array('name'=>'forum_name', 'type' => 'xsd:string', 'nillable' => 'false'),
		'is_public' => array('name'=>'is_public', 'type' => 'xsd:boolean', 'nillable' => 'false'),
		'description' => array('name'=>'description', 'type' => 'xsd:string', 'nillable' => 'true'),
		'send_all_posts_to' => array('name'=>'send_all_posts_to', 'type' => 'xsd:string', 'nillable' => 'true'),
		'thread_count' => array('name'=>'thread_count', 'type' => 'xsd:int', 'nillable' => 'false'),
		'message_count' => array('name'=>'message_count', 'type' => 'xsd:int', 'nillable' => 'false'),
		'most_recent_date' => array('name'=>'most_recent_date', 'type' => 'xsd:string', 'nillable' => 'false'),
		'moderation_level' => array('name'=>'moderation_level', 'type' => 'xsd:int', 'nillable' => 'false'),
		'show_associations_tab' => array('name'=>'show_associations_tab', 'type' => 'xsd:boolean', 'nillable' => 'false'),
		'show_tags_tab' => array('name'=>'show_tags_tab', 'type' => 'xsd:boolean', 'nillable' => 'false'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfForum',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Forum[]')),
	'tns:Forum'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getForums',
	array(
		'sessionHash'=> 'xsd:string',
		'section' => 'xsd:string',
		'ref_id' => 'xsd:int',
		'forum_name' => 'xsd:string',
		'is_public' => 'xsd:int',
		'description' => 'xsd:string',
		'msg_count_min' => 'xsd:int',
		'msg_count_max' => 'xsd:int',
		'recent_date_min' => 'xsd:string',
		'recent_date_max' => 'xsd:string'
	), 
	array('getForums' => 'tns:ArrayOfForum'),
	$uri, 
	$uri.'#getForums',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of Forum
 * @param	$forum_id	Filter by forum_id
 * @param	$section	Filter by section
 * @param	$ref_id	Filter by ref_id
 * @param	$forum_name	Filter by forum_name
 * @param	$is_public	Filter by is_public
 * @param	$description	Filter by description
 * @param	$send_all_posts_to	Filter by send_all_posts_to
 * @param	$thread_count	Filter by thread_count
 * @param	$message_count	Filter by message_count
 * @param	$most_recent_date	Filter by most_recent_date
 * @param	$moderation_level	Filter by moderation_level
 */
function getForums($sessionHash, $section, $ref_id, $forum_name, $is_public, $description, $msg_count_min, $msg_count_max, $recent_date_min, $recent_date_max) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit = new Criteria();
	
	if (!is_null($section) && $section != '') {
		$crit->add(ForumPeer::SECTION, $section);
	}
	if (!is_null($ref_id) && $ref_id > 0) {
		$crit->add(ForumPeer::REF_ID, $ref_id);
	}
	if (!is_null($forum_name) && $forum_name != '') {
		$crit->add(ForumPeer::FORUM_NAME, $forum_name);
	}
	if (!is_null($is_public) && ($is_public==0 || $is_public==1)) {
		$crit->add(ForumPeer::IS_PUBLIC, $is_public==1 ? true:false);
	}
	if (!is_null($description) && $description!='') {
		$crit->add(ForumPeer::DESCRIPTION, "%$description%", Criteria::ILIKE);
	}
	if(!is_null($msg_count_min) && $msg_count_min > -1) {
		$crit->add(ForumPeer::MESSAGE_COUNT, $msg_count_min, Criteria::GREATER_EQUAL);
	}
	if(!is_null($msg_count_max) && $msg_count_max > -1) {
		$crit->add(ForumPeer::MESSAGE_COUNT, $msg_count_max, Criteria::LESS_EQUAL);
	}
	if(!is_null($recent_date_min) && $recent_date_min != '') {
		$crit->add(ForumPeer::MOST_RECENT_DATE, $recent_date_min, Criteria::GREATER_EQUAL);
	}
	if(!is_null($recent_date_max) && $recent_date_max != '') {
		$crit->add(ForumPeer::MOST_RECENT_DATE, $recent_date_max, Criteria::LESS_EQUAL);
	}
	
	$objs = ForumPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = ForumToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getForumArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_id' => 'ArrayOfxsd:int',
	), 
	array('getForum' => 'tns:ArrayOfForum'),
	$uri, 
	$uri.'#getForum',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Forum
 * @param	$forum_id	Retrieve the Forum array identified by the array of forum_id
 */
function getForumArray($sessionHash, $forum_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
        $crit->add(ForumPeer::FORUM_ID,$forum_idArr,Criteria::IN);
	$objs = ForumPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = ForumToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getForum',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_id' => 'xsd:int',
	), 
	array('getForum' => 'tns:Forum'),
	$uri, 
	$uri.'#getForum',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Forum
 * @param	$forum_id	Retrieve the Forum identified by this forum_id
 */
function getForum($sessionHash, $forum_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ForumPeer::retrieveByPK($forum_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getForum','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return ForumToSOAP($obj);
}

$server->register(
	'updateForum',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_id' => 'xsd:int',
		'section' => 'xsd:string',
		'ref_id' => 'xsd:int',
		'forum_name' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'description' => 'xsd:string',
		'send_all_posts_to' => 'xsd:string',
		'moderation_level' => 'xsd:int',
		'show_associations_tab' => 'xsd:boolean',
		'show_tags_tab' => 'xsd:boolean',
	), 
	array('updateForum' => 'xsd:int'),
	$uri, 
	$uri.'#updateForum',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update Forum
 * @param	$forum_id	 Forum to update
 * @param	$section
 * @param	$ref_id
 * @param	$forum_name
 * @param	$is_public
 * @param	$description
 * @param	$send_all_posts_to
 * @param	$thread_count
 * @param	$message_count
 * @param	$most_recent_date
 * @param	$moderation_level
 */
function updateForum($sessionHash, $forum_id, $section, $ref_id, $forum_name, $is_public, $description, $send_all_posts_to, $moderation_level, $show_associations_tab, $show_tags_tab) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ForumPeer::retrieveByPK($forum_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateForum', 'Couldn\'t update Forum: invalid forum_id '.$forum_id);
	}
	
	if (!is_null($section)) {
		$obj->setSection( Validator::sanitizeString($section) );
	}
	if (!is_null($ref_id) && $ref_id != 0) {
		$obj->setRefId( Validator::sanitizeInt($ref_id) );
	}
	if (!is_null($forum_name)) {
		$obj->setForumName( Validator::sanitizeString($forum_name) );
	}
	if (!is_null($is_public)) {
		$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	}
	if (!is_null($description)) {
		$obj->setDescription( Validator::sanitizeString($description) );
	}
	if (!is_null($send_all_posts_to)) {
		$obj->setSendAllPostsTo( Validator::sanitizeString($send_all_posts_to) );
	}
	if (!is_null($moderation_level)) {
		$obj->setModerationLevel( Validator::sanitizeInt($moderation_level) );
	}
	if (!is_null($show_associations_tab)) {
		$obj->setShowAssociationsTab( Validator::sanitizeBool($show_associations_tab) );
	}
	if (!is_null($show_tags_tab)) {
		$obj->setShowTagsTab( Validator::sanitizeBool($show_tags_tab) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateForum','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addForum',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'forum_name' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'description' => 'xsd:string',
		'send_all_posts_to' => 'xsd:string',
		'moderation_level' => 'xsd:int',
		'show_associations_tab' => 'xsd:boolean',
		'show_tags_tab' => 'xsd:boolean',
	), 
	array('addForum' => 'xsd:int'),
	$uri, 
	$uri.'#addForum',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a Forum
 * @param	$section
 * @param	$ref_id
 * @param	$forum_name
 * @param	$is_public
 * @param	$description
 * @param	$send_all_posts_to
 * @param	$thread_count
 * @param	$message_count
 * @param	$most_recent_date
 * @param	$moderation_level
 */
function addForum($sessionHash, $project_id, $forum_name, $is_public, $description, $send_all_posts_to, $moderation_level, $show_associations_tab, $show_tags_tab) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$project_id = Validator::sanitizeInt($project_id);
	$project = ProjectPeer::retrieveByPk($project_id);
	
	if (!$project->userIsAdmin()) {
		return new soap_fault('', 'addForum','Permission Denied - User Is Not Admin');
	}
	
	$obj = new Forum();
	
	$obj->setSection( 'project' );
	$obj->setRefId( $project_id );
	$forum_name = Validator::sanitizeString($forum_name);
	if(is_null($forum_name) || $forum_name=='') {
		return new soap_fault('', 'addForum','Permission Denied - Please Provide a Valid Forum Name ');
	}
	$obj->setForumName( $forum_name );
	if(is_null($is_public)) {
		$is_public = false;
	}
	$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	$obj->setDescription( Validator::sanitizeString($description) );
	$obj->setSendAllPostsTo( Validator::sanitizeString($send_all_posts_to) );
	$obj->setThreadCount( 0 );
	$obj->setMessageCount( 0 );
	$obj->setModerationLevel( Validator::sanitizeInt($moderation_level) );
	$obj->setShowAssociationsTab( Validator::sanitizeBool($show_associations_tab) );
	$obj->setShowTagsTab( Validator::sanitizeBool($show_tags_tab) );
	try {
		$obj->save();
		return $obj->getForumId();
	} catch (Exception $e) {
		return new soap_fault('', 'addForum','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}


$server->register(
	'deleteForum',
	array(
		'sessionHash'	=> 'xsd:string',
		'forum_id' => 'xsd:int',
	), 
	array('deleteForum' => 'xsd:int'),
	$uri, 
	$uri.'#deleteForum',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single Forum
 * @param	$forum_id
 */
function deleteForum($sessionHash, $forum_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ForumPeer::retrieveByPK($forum_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteForum','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	
	if (!$obj->userCanDelete()) {
		return new soap_fault('', 'deleteForum','Permission Denied - Cannot Delete This Record');
	}
	
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteForum','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

function ForumToSOAP(&$obj) {
	$ret = array();
	$ret['forum_id'] = $obj->getForumId();
	$ret['section'] = $obj->getSection();
	$ret['ref_id'] = $obj->getRefId();
	$ret['forum_name'] = $obj->getForumName();
	$ret['is_public'] = $obj->getIsPublic();
	$ret['description'] = $obj->getDescription();
	$ret['send_all_posts_to'] = $obj->getSendAllPostsTo();
	$ret['thread_count'] = $obj->getThreadCount();
	$ret['message_count'] = $obj->getMessageCount();
	$ret['most_recent_date'] = $obj->getMostRecentDate();
	$ret['moderation_level'] = $obj->getModerationLevel();
	$ret['show_associations_tab'] = $obj->getShowAssociationsTab();
	$ret['show_tags_tab'] = $obj->getShowTagsTab();
	return $ret;
}
?>
