<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerItemDuplicate

require_once('plugins/tracker/lib/TrackerItemDuplicate.php');
require_once('plugins/tracker/lib/TrackerItemDuplicatePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerItemDuplicate',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'duplicate_of_tracker_item_id' => array('name'=>'duplicate_of_tracker_item_id', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerItemDuplicate',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerItemDuplicate[]')),
	'tns:TrackerItemDuplicate'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerItemDuplicates',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'duplicate_of_tracker_item_id' => 'xsd:int',
	), 
	array('getTrackerItemDuplicates' => 'tns:ArrayOfTrackerItemDuplicate'),
	$uri, 
	$uri.'#getTrackerItemDuplicates',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerItemDuplicate
 * @param	$tracker_item_id	Filter by tracker_item_id
 * @param	$duplicate_of_tracker_item_id	Filter by duplicate_of_tracker_item_id
 */
function getTrackerItemDuplicates($sessionHash, $tracker_item_id, $duplicate_of_tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$c = new Criteria();
	
	if ($tracker_item_id >0) {
		$c->add(TrackerItemDuplicatePeer::TRACKER_ITEM_ID, $tracker_item_id);
	}
	if ($duplicate_of_tracker_item_id >0) {
		$c->add(TrackerItemDuplicatePeer::DUPLICATE_OF_TRACKER_ITEM_ID, $duplicate_of_tracker_item_id);
	}
	
	$objs = TrackerItemDuplicatePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerItemDuplicateToSOAP($obj);
	}
	
	return $res;
}



$server->register(
	'addTrackerItemDuplicate',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'duplicate_of_tracker_item_id' => 'xsd:int',
	), 
	array('addTrackerItemDuplicate' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerItemDuplicate',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerItemDuplicate
 * @param	$tracker_item_id
 * @param	$duplicate_of_tracker_item_id
 */
function addTrackerItemDuplicate($sessionHash, $tracker_item_id, $duplicate_of_tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = new TrackerItemDuplicate();
	
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setDuplicateOfTrackerItemId( Validator::sanitizeInt($duplicate_of_tracker_item_id) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerItemDuplicate','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}


$server->register(
	'deleteTrackerItemDuplicate',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'duplicate_of_tracker_item_id' => 'xsd:int',
	), 
	array('deleteTrackerItemDuplicate' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerItemDuplicate',
	'rpc',
	'encoded'
);


/**
 * SOAP method to delete a single Duplicate
 * @param	$tracker_item_id
 * @param	$duplicate_of_tracker_item_id
 */
function deleteTrackerItemDuplicate($sessionHash, $tracker_item_id, $duplicate_of_tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$crit= new Criteria();
	$crit->add(TrackerItemDuplicatePeer::TRACKER_ITEM_ID, $tracker_item_id);
	$crit->add(TrackerItemDuplicatePeer::DUPLICATE_OF_TRACKER_ITEM_ID, $duplicate_of_tracker_item_id);

	$obj = TrackerItemDuplicatePeer::doSelectOne($crit);

	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerItemDuplicate','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerItemDuplicate','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


function TrackerItemDuplicateToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['duplicate_of_tracker_item_id'] = $obj->getDuplicateOfTrackerItemId();
	return $ret;
}
?>
